% demo for oMCD: basic check of the expected confidence E[Pc(t)]
% This demo uses Monte-Carlo simulations to validate the semi-analytical
% approximations of 1st- and 2nd-order moments of the absolute difference in
% value modes |dmu(t)| and the ensuing expected confidence (cf. Equations 5
% and 6 of the oMCD manuscript).

clear all
close all
warning off


%% set oMCD parameters
beta = 10; % effort efficacy type #1
gamma = 10; % effort efficacy type #2
Kappa = .01; % effort intensity
m0 = [0.1;-0.1]; % prior mean of value representations
v0 = 16; % prior variance of value representations
maxt = 1/Kappa; % max nb of time samples

% run Monte-Carlo simulations
Nmc = 1e3; % nb of Monte-Carlo simulations
Pc = NaN(maxt,Nmc); % sampled confidence
adm = NaN(maxt,Nmc); % sampled |dmu(t)|
for imc=1:Nmc % loop over Monte-Carlo simulations
    v = NaN(1,maxt); % value representation variance
    m = NaN(2,maxt); % value representation modes
    v(1) = 1./((1/v0)+beta*Kappa);
    m(:,1) = m0 + sqrt(gamma*Kappa)*randn(2,1);
    adm(1,imc) = abs(diff(m(:,1)));
    Pc(1,imc) = getPc(0,0,v(1),m(:,1),0,0);
    for t=2:maxt % loop over time samples
        v(t) = 1./((1/v(t-1))+beta*Kappa);
        m(:,t) = m(:,t-1) + sqrt(gamma*Kappa)*randn(2,1);
        adm(t,imc) = abs(diff(m(:,t)));
        Pc(t,imc) = getPc(0,0,v(t),m(:,t),0,0);
    end
end
[EPc,Eadm,Vadm] = getPc((1:maxt)*Kappa,beta,v0,m0,gamma,0); % approx
hf = figure('name','demo: E[confidence] in online-MCD');
ha = subplot(2,2,1,'parent',hf,'nextplot','add');
madm = mean(adm,2); % MC estimate of E[|dmu(t)|]
plot(ha,madm,'k')
plot(ha,Eadm,'r')
xlabel(ha,'time (a.u.)')
ylabel(ha,'E[|dmu(t)|]')
ha = subplot(2,2,2,'parent',hf,'nextplot','add');
vadm = var(adm,[],2); % MC estimate of Var[|dmu(t)|]
plot(ha,vadm,'k')
plot(ha,Vadm,'r')
xlabel(ha,'time (a.u.)')
ylabel(ha,'V[|dmu(t)|]')
ha = subplot(2,2,3,'parent',hf,'nextplot','add');
mp = mean(Pc,2); % MC estimate of E[Pc(t)]
plot(ha,mp,'k')
plot(ha,EPc,'r')
xlabel(ha,'time (a.u.)')
ylabel(ha,'E[Pc(t)]')
