% This script checks whether prospective MCD and oMCD yields similar
% predictions (across various decision parameter settings).
% This demo reproduces Figure 5 of Benon et al. (2024).

clear all
close all
warning off

%% set fixed MCD parameters
maxz = 1; % max amount of resources
T = 1e2; % nb of time samples
kappa = maxz/T; % rate of resource expanditure per unit of time
ts = 1:T; % temporal grid

%% run Monte-Carlo simulations
Nmc = 2e2; % nb of MDC parameter settings
N = 5e2; % nb of sample path trajectories (for deriving averages)

Z = NaN(Nmc,2,N); % prospective MCD: invested resources
Pc = NaN(Nmc,2,N); % prospective MCD: ensuing confidence levels
Q = NaN(Nmc,2,N); % oMCD: mean discounted benefits
gammas = NaN(Nmc,1); % sampled type #2 effort efficacies

for imc=1:Nmc % loop through parameter settings
    
    imc
    % set varying MCD parameters
    nu = 8*rand; % effort cost power
    R = 8*rand; % decision importance
    beta = 8*rand; % effort efficiency type #1
    gamma = 8*rand; % effort efficiency type #2
    alpha = 8*rand; % unitary cost of effort
    v0 = 8*rand; % initial (pre-decision) value variance
    m0 = 8*rand(2,1); % initial value modes
    
    % prospective MCD: resource investment
    EPc = getPc(kappa*ts,beta,v0,m0,gamma,0); % anticipated confidence
    EQ0t = R*EPc - alpha*(kappa*ts).^nu; % anticipated discounted benefits
    tmcd = find(EQ0t==max(EQ0t),1); % best discounted value
    Z(imc,2,:) = kappa*tmcd; % ensuing amount of invested resources
    
    % get oMCD optimal threshold
    [w,wP] = get_wMDP0(R,alpha,beta,gamma,kappa,nu,v0);
    
    % dynamics of value representations: Monte-Carlo simulations
    Pct = NaN(N,T); % sample paths of confidence
    Q0t = NaN(N,T); % sample paths of discounted benefits
    for i=1:N % loop over Monte-Carlo simulations
        v = NaN(1,T); % variance of value representations
        m = NaN(2,T); % mean of value representations
        v(1) = 1./((1/v0)+beta*kappa); % 1st updated variance
        m(:,1) = m0 + sqrt(gamma*kappa)*randn(2,1); % 1st updated mean
        Pct(i,1) = getPc(0,0,v(1),m(:,1),0,0); % % 1st updated confidence
        Q0t(i,1) = R*Pct(i,1) - alpha*kappa.^nu; % 1st updated discounted benefit
        for t=2:T % loop over time samples
            v(t) = 1./((1/v(t-1))+beta*kappa); % update variance
            m(:,t) = m(:,t-1) + sqrt(gamma*kappa)*randn(2,1); % update modes
            Pct(i,t) = getPc(0,0,v(t),m(:,t),0,0); % evaluate current confidence
            Q0t(i,t) = R*Pct(i,t) - alpha*(kappa*t).^nu; % discounted benfit
        end
        % Detect oMCD decision properties
        tomcd = find(Pct(i,:)>=wP,1); % 1st hitting time
        Z(imc,1,i) = kappa*tomcd;
        Pc(imc,1,i) = Pct(i,tomcd);
        Q(imc,1,i) = Q0t(i,tomcd);
        % Detect MCD decision properties
        Pc(imc,2,i) = Pct(i,tmcd);
        Q(imc,2,i) = Q0t(i,tmcd);
    end
    
    % store type #2 effort efficacy
    gammas(imc) = gamma;

    
end


x = sig(3*(gammas-mean(gammas)));
cols = [x,zeros(Nmc,1),1-x];

hf = figure('name','Prospective/online MCD'); % Figure 5
titles = {'A: resource investment','B: confidence','C: net benefit'};
for i=1:3
    ha(i) = subplot(2,2,i,'parent',hf,'nextplot','add');
    title(ha(i),titles{i})
end
for imc=1:Nmc
    plot(ha(1),mean(Z(imc,2,:),3),mean(Z(imc,1,:),3),'.','color',cols(imc,:))
    plot(ha(2),mean(Pc(imc,2,:),3),mean(Pc(imc,1,:),3),'.','color',cols(imc,:))
    plot(ha(3),mean(Q(imc,2,:),3),mean(Q(imc,1,:),3),'.','color',cols(imc,:))
end
set(ha(1),'xlim',[0,1],'ylim',[0,1]);
set(ha(2),'xlim',[0.4,1],'ylim',[0.4,1]);
for i=1:3
    plot(ha(i),get(ha(i),'xlim'),get(ha(i),'xlim'),'k--')
    xlabel(ha(i),'prospective MCD policy')
    ylabel(ha(i),'oMCD policy')
end
