% This script eyeballs intermediate steps in the derivation of oMCD control
% policy.
% This reproduces Figure 1 in Benon et al. (2024)

clear all
close all
warning off

%% Set default MCD parameters
nu = 0.5; % effort cost power
R = 1; % decision importance
beta = 1; % type #1 effort efficacy
gamma = 4; % type #2 effort efficacy
alpha = 0.2; % unitary cost of effort
v0 = 1; % initial (pre-decision) value variance
m0 = [0;0]; % initial value modes
maxz = 1; % max amount of resources
T = 1e2; % nb of time samples
Kappa = maxz/T; % rate of resource expanditure per unit of time
zs = Kappa:Kappa:1; % admissible resource amounts


%% Derive oMCD optimal threshold
[wMDP0,wp0,wdmu,Pt,dmut] = get_wMDP0(R,alpha,beta,gamma,Kappa,nu,v0);

%% eyeball net benefits
hf = figure('name','oMCD optimal stopping strategy: example net benefits');
% plot net benefits and ther expectations
ha = axes('parent',hf,'nextplot','add');
i0 = find(dmut==0);
vTm1 = 1/((1/v0)+beta*Kappa*(T-1)); % value variance at T-1
QTm1 = R*getPc_dmu(0,0,vTm1,dmut,0,0) - alpha*(Kappa*(T-1))^nu; % net benefit at t=T-1
plot(ha,dmut(i0:end),QTm1(i0:end),'r')
EQT = R*getPc_dmu(Kappa,beta,vTm1,dmut,gamma,0) - alpha*(Kappa*T)^nu; % E[net benefit at t=T]
plot(ha,dmut(i0:end),EQT(i0:end),'g')
iu = find(QTm1(i0:end)>=EQT(i0:end),1);
dmu_star = dmut(i0+iu-1);
w_star = QTm1(i0+iu-1);
Qstar = max([QTm1,EQT],[],2); % optimal net benefit at t=T-1
plot(ha,dmut(i0:end),Qstar(i0:end),'k--')
EQTm1 = Pt*Qstar; % E[optimal net benefit at t=T-1]
plot(ha,dmut(i0:end),EQTm1(i0:end),'b--')
plot(ha,dmu_star,w_star,'k.')
plot(ha,[0,dmu_star],w_star*[1,1],'k--')
plot(ha,dmu_star*[1,1],[0,w_star],'k--')
xlabel(ha,'\Delta\mu')
ylabel(ha,'net benefit')
legend(ha,{...
    'Q(0,\Delta\mu(T-1)=\Delta\mu)',...
    'E[Q(0,\Delta\mu(T))|\Delta\mu(T-1)=\Delta\mu]',...
    'Q^*(\Delta\mu(T-1)=\Delta\mu)',...
    'E[Q^*(\Delta\mu(T-1))|\Delta\mu(T-2)=\Delta\mu]'})
if w_star>0
    set(ha,'xlim',[0,3],'ytick',[0,w_star],'yticklabel',{'0','\omega^*(T-1)'})
else
    set(ha,'xlim',[0,3],'ytick',[w_star,0],'yticklabel',{'\omega^*(T-1)','0'})
end
