% This demo compares oMCD with the approach in Tajima et al (2016).
% More precisely, this demo compares the optimal stopping policies of the
% two candidate settings of the rsource allocation problem, which
% correspond to defining the benefit term as either (i) decision confidence
% or (ii) the value of the chosen option. The former is oMCD, while the
% latter is the approach in Tajima et al. (2024).
% This reproduces Figure 9 and A3 of Benon et al. (2024).

clear all
close all
warning off


%% Eyeball summary statitics of E[max(X)]

X = randn(2,1e3);
M = max(X,[],1);
t1 = mean(X,1);
t2 = abs(diff(X,[],1))/2;
hf = figure;
ha = subplot(2,2,1,'parent',hf,'nextplot','add');
plot(ha,M,t1+t2,'k.')
plot(ha,get(ha,'xlim'),get(ha,'xlim'),'r--')
xlabel(ha,'max(X_1,X_2)')
ylabel(ha,'0.5*(X_1+X_2) + 0.5*|\DeltaX|')
ha = subplot(2,2,2,'parent',hf);
plot(ha,t1,t2,'k.')
xlabel(ha,'0.5*(X_1+X_2)')
ylabel(ha,'0.5*|\DeltaX|')

N = 1e3;
Nmc = 100;
for i=1:Nmc
    % sample E[max(X)]
    mu = randn(2,1);
    A = randn(2,2);
    V = A'*A;
    S = sqrtm(V);
    X = repmat(mu,1,N) + S*randn(2,N);
    M = max(X,[],1);
    mM(i) = mean(M);
    % derive analytical expression
    adm = abs(diff(mu));
    vdm = V(1,1) + V(2,2) - 2*V(1,2);
    tmp = adm./sqrt(vdm);
    mM0(i) = mean(mu) + sqrt(vdm./(2*pi)).*exp(-0.5*tmp.^2 ) + 0.5*adm.*(1-2*normcdf(-tmp));
end
ha = subplot(2,2,3,'parent',hf,'nextplot','add');
plot(ha,mM,mM0,'k.')
plot(ha,get(ha,'xlim'),get(ha,'xlim'),'r--')
xlabel(ha,'E[max(X)]: Monte-Carlo')
ylabel(ha,'E[max(X)]: analytical')



%% look at value for waiting vs value for stopping (at t=T-1)

% set MCD parameters
alpha = 1; % effort cost sensitivity
beta = 1; % type #1 effort efficacy
gamma = 1; % type #2 effort efficacy
nu = 1; % effort cost power
kappa = .05; % effort intensity
R = 1; % decision importance
v0 = .1; % prior value variance
T = 1/kappa; % temporal horizon

% estimates costs
C0 = alpha*(kappa*(T-1)).^nu; % effort cost at t=T-1
CT = alpha*(kappa*T).^nu; % effort cost at t=T

% net benefits at t=T-1 (chosen option's value)
gamkap = gamma*kappa;
dmu = -(8*sqrt(2)*gamkap):1e-4:(8*sqrt(2)*gamkap); % admissible dmu(T-1)
Q_stop = 0.5*abs(dmu) - C0; % stop
tmp = abs(dmu)./sqrt(gamkap);
Edmu = 2*sqrt(gamkap./(2*pi)).*exp(-0.5*tmp.^2 ) + abs(dmu).*(1-2*normcdf(-tmp)); % E[dmu(T)|dmu(T-1)]
Q_wait =  0.5*Edmu - CT; % wait

% net benefits at t=T-1 (oMCD)
Q_stop0 = R*getPc_dmu(0,beta,v0,dmu,gamma,0) - C0; % stop
Q_wait0 = R*getPc_dmu(kappa,beta,v0,dmu,gamma,0) - CT; % wait


% compare net benefits as a function of dmu(T-1)
hf = figure; % Figure A3
ha = subplot(2,2,1,'parent',hf,'nextplot','add');
plot(ha,dmu,Q_stop,'r')
plot(ha,dmu,Q_wait,'g')
xlabel(ha,'\Delta\mu')
ylabel(ha,'net benefit')
legend(ha,{'stop','wait'})
title(ha,'chosen option''s value')
ha = subplot(2,2,2,'parent',hf,'nextplot','add');
plot(ha,dmu,Q_stop0,'r')
plot(ha,dmu,Q_wait0,'g')
xlabel(ha,'\Delta\mu')
ylabel(ha,'net benefit')
legend(ha,{'stop','wait'})
title(ha,'confidence (oMCD)')


% compare thresholds
beta = 1; % type #1 effort efficacy
gamma = 1; % type #2 effort efficacy
kappa = .01; % effort intensity
R = 100; % decision importance
alpha = 0.3; % effot cost sensitivity
nu = 1; % effot cost power
v0 = 1; % prior value variance
T = 1/kappa; % temporal horizon

[w,wP,wdmu0] = get_wMDP0(R,alpha,beta,gamma,kappa,nu,v0);
[w,wdmu] = get_wMDP_Tajima(alpha,gamma,kappa,nu);

ha = subplot(2,2,3,'parent',hf,'nextplot','add');
plot(ha,wdmu,'k')
plot(ha,wdmu0,'k--')
xlabel(ha,'decision time')
ylabel(ha,'\omega_{\Delta}(t)')
legend(ha,{'chosen option''s value','confidence (oMCD)'})
title(ha,'optimal |\Delta\mu| threshold')

% compare resource investments
N = 1e4; % nb of simulated sample paths
dmut = cumsum(sqrt(2*gamkap)*randn(N,T),2); % sample paths of dmu(t)
Z = NaN(N,2);
for imc=1:N % loop through MC simulations
    i0 = find(abs(dmut(imc,:))>=wdmu0,1);
    i1 = find(abs(dmut(imc,:))>=wdmu,1);
    Z(imc,2) = kappa*i0; % oMCD
    Z(imc,1) = kappa*i1; % Tajima
end
[py,gridy] = empiricalHist(Z,.1);
ha = subplot(2,2,4,'parent',hf,'nextplot','add');
hp = plot(ha,gridy,py,'k');
set(hp(1),'linestyle','-')
set(hp(2),'linestyle','--')
xlabel(ha,'invested resource (z)')
ylabel(ha,'P(z|policy)')
title(ha,'resource investment')
legend(ha,{'chosen option''s value','confidence (oMCD)'})
set(ha,'ytick',[],'xlim',[0,1])



%% systematic variations induced by MCD parameters
Nmc = 2e2; % nb of simulated parameter sets
N = 5e2; % nb of simulated sample paths
kappa = .01; % effort intensity
R = 1; % decision importance
T = 1/kappa; % temporal horizon

betas = NaN(Nmc,1);
Z = NaN(Nmc,2,N);
C = NaN(Nmc,2,N);
Q0 = NaN(Nmc,2,N);
Q1 = NaN(Nmc,2,N);
for imc=1:Nmc % loop over MC simulations
    imc
    % sample MCD parameters
    beta = abs(randn); % type #1 effort efficacy
    gamma = abs(randn); % type #2 effort efficacy
    alpha = abs(randn); % effort cost sensitivity
    nu = abs(randn); % effort cost power
    v0 = abs(randn); % prior value variance
    % derive optimal stopping policies
    [w,wP,wdmu0] = get_wMDP0(R,alpha,beta,gamma,kappa,nu,v0);
    [w,wdmu] = get_wMDP_Tajima(alpha,gamma,kappa,nu);
    % simulate sample path trajectories
    mu1 = cumsum(sqrt(gamkap)*randn(N,T),2); % 1st option
    mu2 = cumsum(sqrt(gamkap)*randn(N,T),2); % 2nd option
    dmut = mu1-mu2; % value mode difference
    maxV = max(cat(3,mu1,mu2),[],3); % chosen option's value
    % derive mean RT, confidence and value
    for i=1:N % loop through sample path trajectories
        % oMCD stopping policy
        i0 = find(abs(dmut(i,:))>=wdmu0,1); % index of 1st hitting time
        Z(imc,1,i) = kappa*i0; % invested resource
        dmu = dmut(i,i0); % corresponding dmu
        vt = 1/((1/v0)+beta*Z(imc,1,i)); % value variance
        C(imc,1,i) = getPc_dmu(0,0,vt,dmu,0,0); % confidence
        Q0(imc,1,i) = R*C(imc,1,i) - alpha*Z(imc,1,i)^nu; % confidence
        Q1(imc,1,i) = 0.5*abs(dmu)- alpha*Z(imc,1,i)^nu; % chosen option value
        % Tajima's stopping policy
        i1 = find(abs(dmut(i,:))>=wdmu,1); % index of 1st hitting time
        Z(imc,2,i) = kappa*i1; % invested resource
        dmu = dmut(i,i1); % corresponding dmu
        vt = 1/((1/v0)+beta*Z(imc,2,i)); % value variance
        C(imc,2,i) = getPc_dmu(0,0,vt,dmu,0,0); % confidence
        Q0(imc,2,i) = R*C(imc,2,i) - alpha*Z(imc,2,i)^nu; % confidence
        Q1(imc,2,i) = 0.5*abs(dmu)- alpha*Z(imc,2,i)^nu; % chosen option value
    end
    
    % store type #1 effort efficacy
    betas(imc) = beta;
    
end

x = sig(3*(betas-mean(betas)));
cols = [x,zeros(Nmc,1),1-x];

hf = figure; % Figure 9
titles = {'A: resource investments','B: confidence','C: net benefit (oMCD)','D: net benefit (max(value))'};
for i=1:4
    ha(i) = subplot(2,2,i,'parent',hf,'nextplot','add');
    title(ha(i),titles{i})
end
for imc=1:Nmc
    plot(ha(1),mean(Z(imc,2,:),3),mean(Z(imc,1,:),3),'.','color',cols(imc,:))
    plot(ha(2),mean(C(imc,2,:),3),mean(C(imc,1,:),3),'.','color',cols(imc,:))
    plot(ha(3),mean(Q0(imc,2,:),3),mean(Q0(imc,1,:),3),'.','color',cols(imc,:))
    plot(ha(4),mean(Q1(imc,2,:),3),mean(Q1(imc,1,:),3),'.','color',cols(imc,:))
end
set(ha(1),'xlim',[0,1],'ylim',[0,1]);
set(ha(2),'xlim',[0.4,1],'ylim',[0.4,1]);
for i=1:4
    plot(ha(i),get(ha(i),'xlim'),get(ha(i),'xlim'),'k--')
    xlabel(ha(i),'max(value) policy')
    ylabel(ha(i),'oMCD policy')
end


