% oMCD demo: scenarios for value belief update
% This script simulates oMCD control applied on two distinct value update
% scenarios: (i) bayesian value denoising, and (ii) progressive
% attribute integration. These secnarios differ in the way value
% representations are updated over time, but both yield confidence levels
% that can be monitored, anticipated, and optimally thresholded.
% The script then evaluates the properties of the statistical relationships
% that exist between response time, confidence and value-consistency of
% choice, under oMCD's policy.
% This reproduces Figures 7 and 8 of Benon et al. (2024)
% Note on functional dependency:
% - requires VBA_logisticReg.m
% - requires VBA_plotxy.m


clear all
close all
warning off

%% Bayesian value denoising

% set ideal observer parameters
v0 = 10; % initial (pre-decision) value variance
m0 = [0;0]; % initial value modes
S = 1e3; % noise variance
T = 1e2; % temporal horizon (nb of time samples)

% Set MCD parameters
R = 1; % decision importance
alpha = .1; % unitary cost of effort
nu = 2; % effort cost power
maxz = 1; % max amount of resources
Kappa = maxz/T; % rate of resource expanditure per unit of time
% Derive effort efficacies from ideal observer parameters
beta = 1/(Kappa*S); %  type #1 effort efficacy
ts = 1:T; % time indices
gamma = mean((S+ts.*mean(v0))./((S./v0)+ts).^2)/Kappa; % type #2 effort efficacy
% Get oMCD optimal threshold process
[wMDP0,wp0] = get_wMDP0(R,alpha,beta,gamma,Kappa,nu,v0);

% Perform Monte-Carlo simulations
Nmc = 1e4;
Pct = NaN(Nmc,T);
t_omcd = NaN(Nmc,1);
P_omcd = NaN(Nmc,1);
c_omcd = NaN(Nmc,1);
dV = NaN(Nmc,1);
for imc=1:Nmc
    imc
    % simulate noisy input signals
    V = sqrt(v0)*randn(2,1); % hidden value signal
    e = sqrt(S)*randn(2,T);
    y = repmat(V,1,T) + e;
    % ideal observer: 1st time step
    m = NaN(2,T);
    v = NaN(2,T);
    v(:,1) = 1./((1/v0)+(1/S)); % update variance
    d(:,1) = (y(:,1)-m0)./((S./v0)+1);
    m(:,1) = m0 + d(:,1);
    Pct(imc,1) = getPc(0,0,v(:,1),m(:,1),0,0); % current confidence
    % ideal observer: all other time steps
    for t=2:T
        d(:,t) = sum(y(:,1:t)-repmat(m0,1,t),2)./((S./v0)+t);
        v(:,t) = 1./((1/v0)+(t/S)); % update variance
        m(:,t) = m0 + d(:,t);
        Pct(imc,t) = getPc(0,0,v(:,t),m(:,t),0,0); % current confidence
    end
    dV(imc) = diff(V);
    t_omcd(imc) = find(Pct(imc,:)>=wp0,1); % oMCD: response time
    P_omcd(imc) = Pct(imc,t_omcd(imc)); % oMCD: confidence
    c_omcd(imc) = diff(m(:,t_omcd(imc)))>0; % ideal observer's choice
end

% Eyeball simulation results
hf = figure('name','Bayesian value denoising'); % Figure 7
% plot confidence dynamics
ha = subplot(2,2,1,'parent',hf,'nextplot','add');
mp = mean(Pct); % average confidence at time t
plot(ha,mp,'b')
plot(ha,wp0,'k--')
e05 = prctile(Pct,5,1);
e95 = prctile(Pct,95,1);
mtmp = mean([e05;e95],1);
vtmp = (mtmp-e95).^2;
[~,~,hp] = plotUncertainTimeSeries(mtmp,vtmp,[],ha,1,'b');
delete(hp)
EPc = getPc(Kappa:Kappa:1,beta,v0,m0,gamma,0); % anticipated confidence
plot(ha,EPc,'b--')
set(ha,'xtick',[],'ytick',[])
xlabel(ha,'resource')
ylabel(ha,'confidence')
title(ha,'A: oMCD policy')

% plot effort investments
ha = subplot(2,2,2,'parent',hf,'nextplot','add');
out = VBA_plotxy(dV,Kappa*t_omcd,8,ha,1);
set(out.he,'color','k')
mep = median(P_omcd);
ih = find(P_omcd>=mep); % high confidence trials
out = VBA_plotxy(dV(ih),Kappa*t_omcd(ih),8,ha,1);
set(out.he,'color','b')
il = find(P_omcd<=mep); % low confidence trials
out = VBA_plotxy(dV(il),Kappa*t_omcd(il),8,ha,1);
set(out.he,'color','r')
set(ha,'ylim',[0.5,1.01])
legend(ha,{'all trials','high confidence','low confidence'})
xlabel(ha,'dv')
ylabel(ha,'resource investment')
title(ha,'B: resource investment')

% plot choice
ha = subplot(2,2,3,'parent',hf,'nextplot','add');
b = VBA_logisticReg(c_omcd,[dV,ones(Nmc,1)]);
dvs = linspace(min(dV),max(dV),1e2);
pcs = feval(@g_logistic,[],b,[],struct('X',[vec(dvs),ones(length(dvs),1)]));
plot(ha,dvs,pcs,'k')
b = VBA_logisticReg(c_omcd(ih),[dV(ih),ones(length(ih),1)]);
dvs = linspace(min(dV),max(dV),1e2);
pcs = feval(@g_logistic,[],b,[],struct('X',[vec(dvs),ones(length(dvs),1)]));
plot(ha,dvs,pcs,'b')
b = VBA_logisticReg(c_omcd(il),[dV(il),ones(length(il),1)]);
dvs = linspace(min(dV),max(dV),1e2);
pcs = feval(@g_logistic,[],b,[],struct('X',[vec(dvs),ones(length(dvs),1)]));
plot(ha,dvs,pcs,'r')
legend(ha,{'all trials','high confidence','low confidence'})
xlabel(ha,'dv')
ylabel(ha,'P(choice)')
title(ha,'C: value consistency')

% plot confidence
ha = subplot(2,2,4,'parent',hf,'nextplot','add');
out = VBA_plotxy(dV,P_omcd,8,ha,1);
set(out.he,'color','k')
cc = c_omcd==(dV>0); % value consistency
ic = find(cc==1); % choice-consistent trials
out = VBA_plotxy(dV(ic),P_omcd(ic),8,ha,1);
set(out.he,'color','b')
ii = find(cc==0); % value-inconsistent trials
out = VBA_plotxy(dV(ii),P_omcd(ii),8,ha,1);
set(out.he,'color','r')
pc = num2str(100*length(ic)/Nmc,3);
pi = num2str(100*length(ii)/Nmc,3);
set(ha,'ylim',[0.5,1])
legend(ha,{'all trials',['value-consistent (',pc,'%)'],['value-inconsistent (',pi,'%)']})
xlabel(ha,'dv')
ylabel(ha,'confidence')
title(ha,'D: achieved confidence')



%% Progressive attribute integration

% set attribute-integration parameters
k = 20; % nb of attributes (=nb of decision time steps)
eta = zeros(k,1); % attribute means
zeta = 1*abs(ones(k,1)); % attribute variances
w = ones(k,1); % attribute weights

% set motivational parameters
nu = 4; % effort cost power
R = 1; % decision importance
alpha = 3; % unitary cost of effort
% Set MCD parameters
v0 = (w.^2)'*zeta; % prior value variance
m0 = w'*eta*[1;1]; % prior value mean
maxz = 1; % max amount of resources
Kappa = maxz/k; % rate of resource expanditure per unit of time
% Derive effort efficacies from attribute-integration model parameters
ts = 1:(k-1);
beta = mean(1./(k-ts))/(Kappa*v0); % effort efficiency type #1
gamma = v0;
% Get oMCD optimal threshold process
[wMDP0,wp0] = get_wMDP0(R,alpha,beta,gamma,Kappa,nu,v0);

% Perform Monte-Carlo simulations
Nmc = 1e4;
Pct = NaN(Nmc,k);
t_omcd = NaN(Nmc,1);
P_omcd = NaN(Nmc,1);
c_omcd = NaN(Nmc,1);
dV = NaN(Nmc,1);
dm = NaN(Nmc,1);
for imc=1:Nmc
    imc
    % random permutation order of attribute disclosing
    ia = randperm(k);
    % 1st option: sample attributes
    x1 = eta + sqrt(zeta).*randn(k,1);
    % 2nd option: sample attributes
    x2 = eta + sqrt(zeta).*randn(k,1);
    x = [x1';x2']; % options attributes
    V = x*w;
    
    % attribute-integration: 1st time step
    m = NaN(2,k);
    v = NaN(2,k);
    v(:,1) = v0 - w(ia(1)).^2*zeta(ia(1)); % update variance
    d(:,1) = w(ia(1))*(x(:,ia(1))-eta(ia(1)));
    m(:,1) = m0 + d(:,1);
    Pct(imc,1) = getPc(0,0,v(:,1),m(:,1),0,0); % current confidence
    % ideal observer: all other time steps
    for t=2:k
        d(:,t) = (x(:,ia(1:t))-repmat(eta(ia(1:t))',2,1))*w(ia(1:t));
        v(:,t) = v0 - sum(w(ia(1:t)).^2.*zeta(ia(1:t))); % update variance
        m(:,t) = m0 + d(:,t);
        Pct(imc,t) = getPc(0,0,v(:,t),m(:,t),0,0); % current confidence
    end
    Pct(imc,k) = 1; % last trial
    dV(imc) = diff(V);
    dm(imc) = diff(m(:,k));
    t_omcd(imc) = find(Pct(imc,:)>=wp0,1); % oMCD: response time
    P_omcd(imc) = Pct(imc,t_omcd(imc)); % oMCD: confidence
    c_omcd(imc) = diff(m(:,t_omcd(imc)))>0; % ideal observer's choice
end

%% Eyeball simulation results
hf = figure('name','attribute integration case'); % Figure 8
% plot confidence dynamics
ha = subplot(2,2,1,'parent',hf,'nextplot','add');
mp = mean(Pct); % average confidence at time t
plot(ha,mp,'b')
plot(ha,wp0,'k--')
e05 = prctile(Pct,5,1);
e95 = prctile(Pct,95,1);
mtmp = mean([e05;e95],1);
vtmp = (mtmp-e95).^2;
[~,~,hp] = plotUncertainTimeSeries(mtmp,vtmp,[],ha,1,'b');
delete(hp)
EPc = getPc(Kappa:Kappa:1,beta,v0,m0,gamma,0); % anticipated confidence
plot(ha,EPc,'b--')
set(ha,'xtick',[],'ytick',[])
xlabel(ha,'resource')
ylabel(ha,'confidence')
title(ha,'A: oMCD policy')

% plot RT
ha = subplot(2,2,2,'parent',hf,'nextplot','add');
out = VBA_plotxy(dV,Kappa*t_omcd,8,ha,1);
set(out.he,'color','k')
mep = median(P_omcd);
ih = find(P_omcd>=mep); % high confidence trials
out = VBA_plotxy(dV(ih),Kappa*t_omcd(ih),8,ha,1);
set(out.he,'color','b')
il = find(P_omcd<=mep); % low confidence trials
out = VBA_plotxy(dV(il),Kappa*t_omcd(il),8,ha,1);
set(out.he,'color','r')
set(ha,'ylim',[0.15,0.5])
legend(ha,{'all trials','high confidence','low confidence'})
xlabel(ha,'dv')
ylabel(ha,'resource investment')
title(ha,'B: resource investment')


% plot choice
ha = subplot(2,2,3,'parent',hf,'nextplot','add');
b = VBA_logisticReg(c_omcd,[dV,ones(Nmc,1)]);
dvs = linspace(min(dV),max(dV),1e2);
pcs = feval(@g_logistic,[],b,[],struct('X',[vec(dvs),ones(length(dvs),1)]));
plot(ha,dvs,pcs,'k')
b = VBA_logisticReg(c_omcd(ih),[dV(ih),ones(length(ih),1)]);
dvs = linspace(min(dV),max(dV),1e2);
pcs = feval(@g_logistic,[],b,[],struct('X',[vec(dvs),ones(length(dvs),1)]));
plot(ha,dvs,pcs,'b')
b = VBA_logisticReg(c_omcd(il),[dV(il),ones(length(il),1)]);
dvs = linspace(min(dV),max(dV),1e2);
pcs = feval(@g_logistic,[],b,[],struct('X',[vec(dvs),ones(length(dvs),1)]));
plot(ha,dvs,pcs,'r')
legend(ha,{'all trials','high confidence','low confidence'})
xlabel(ha,'dv')
ylabel(ha,'P(choice)')
title(ha,'C: value consistency')

% plot confidence
ha = subplot(2,2,4,'parent',hf,'nextplot','add');
out = VBA_plotxy(dV,P_omcd,8,ha,1);
set(out.he,'color','k')
cc = c_omcd==(dV>0); % value consistency
ic = find(cc==1); % choice-consistent trials
out = VBA_plotxy(dV(ic),P_omcd(ic),8,ha,1);
set(out.he,'color','b')
ii = find(cc==0); % value-inconsistent trials
out = VBA_plotxy(dV(ii),P_omcd(ii),8,ha,1);
set(out.he,'color','r')
pc = num2str(100*length(ic)/Nmc,3);
pi = num2str(100*length(ii)/Nmc,3);
set(ha,'ylim',[0.7,0.9])
legend(ha,{'all trials',['value-consistent (',pc,'%)'],['value-inconsistent (',pi,'%)']})
xlabel(ha,'dv')
ylabel(ha,'confidence')
title(ha,'D: achieved confidence')
