% oMCD demo: effect of decision difficulty and reward (under ideal observer
% secnario)
clear all
close all

%% Set simulation parameters

% set ideal observer parameters
v0 = 1e1; % initial (pre-decision) value variance
m0 = [1;-1]; % initial value modes (-> default preference)
S = 1e2;%1e1 % noise variance
nt = 60; % nb of time samples

% Set fixed MCD parameters
pow = 2;%1;
alpha = 2;%2; % unitary cost of effort
maxz = 1; % max amount of resources
Kappa = maxz/nt; % rate of resource expanditure per unit of time
beta = 1/(Kappa*S); % effort efficiency type #1
ts = 1:nt; % admissible decision times
gamma = mean((S+ts.*mean(v0))./((S./v0)+ts).^2)/Kappa;

% Set task parameters
ds = 0:.2:1; % decision difficulty
Rs = [1,3]; % rewards

% Initialize simulation variables
Nmc = 1e3;
rt = NaN(length(Rs),length(ds),Nmc);
side = NaN(length(Rs),length(ds),Nmc);
cue = NaN(length(Rs),length(ds),Nmc);
dif = NaN(length(Rs),length(ds),Nmc);
rew = NaN(length(Rs),length(ds),Nmc);
conf = NaN(length(Rs),length(ds),Nmc);

%% Run simulations
hf = figure('name','oMCD: decimotiv');
cols = 'br';
lss = {'-','--'};
ha(1) = subplot(3,2,1,'parent',hf,'nextplot','add');
title(ha(1),'confidence threshold')
xlabel(ha(1),'decision time')
ylabel(ha(1),'confidence')
ha(2) = subplot(3,2,2,'parent',hf,'nextplot','add');
title(ha(2),'metacognitive sensitivity')
xlabel(ha(2),'confidence')
ylabel(ha(2),'accuracy')

for ir=1:length(Rs)
    % set oMCD optimal confidence threshold
    R = Rs(ir);
    [wMDP0,wp0] = get_wMDP0(R,alpha,beta,gamma,Kappa,pow,v0,nt);
    plot(ha(1),ts,wp0,'color',cols(ir))
    for id=1:length(ds)
        d = ds(id); % difficulty
        Pct = NaN(Nmc,nt);
        % Perform Monte-Carlo simulations
        for imc=1:Nmc
            % simulate noisy input signals
            V = sign(randn)*[d/2;1-d/2]; % hidden value signal
            e = sqrt((1+d)*S)*randn(2,nt);
            y = repmat(V,1,nt) + e;
            % ideal observer: 1st time step
            m = NaN(2,nt);
            v = NaN(2,nt);
            v(:,1) = 1./((1/v0)+(1/S)); % update variance
            delta(:,1) = (y(:,1)-m0)./((S./v0)+1);
            m(:,1) = m0 + delta(:,1);
            Pct(imc,1) = getPc(0,0,v(:,1),m(:,1),0,0); % current confidence
            % ideal observer: all other time steps
            for t=2:nt
                delta(:,t) = sum(y(:,1:t)-repmat(m0,1,t),2)./((S./v0)+t);
                v(:,t) = 1./((1/v0)+(t/S)); % update variance
                m(:,t) = m0 + delta(:,t);
                Pct(imc,t) = getPc(0,0,v(:,t),m(:,t),0,0); % current confidence
            end
            % oMCD: decision time, confidence and choice
            t_omcd = find(Pct(imc,:)>=wp0,1); % oMCD: response time
            P_omcd = Pct(imc,t_omcd); % oMCD: confidence
            c_omcd = sign(-diff(m(:,t_omcd))); % ideal observer's choice
            % store simulated trial
            rt(ir,id,imc) = t_omcd;
            side(ir,id,imc) = c_omcd;
            cue(ir,id,imc) = sign(-diff(V)+1e-3*randn);
            dif(ir,id,imc) = d; % difficulty level
            rew(ir,id,imc) = R;
            conf(ir,id,imc) = P_omcd;
        end
        mpc = mean(Pct,1);
        if ir==2
            if id==1
               plot(ha(1),ts,mpc,'k','linestyle','-')
            elseif id==6
                plot(ha(1),ts,mpc,'k','linestyle','--')
                legend(ha(1),{'\omega_P(R=1)','\omega_P(R=3)','conf(0:10)','conf(5:5)'})
            end
        end
    end
    acc = (cue==side);
    out = VBA_plotxy(vec(conf(ir,:,:)),vec(acc(ir,:,:)),5,ha(2),1);
    set(out.he,'color',cols(ir))
end


%% Eyeball simulation results
% plot accuracy
ha = subplot(3,2,3,'parent',hf,'nextplot','add');
acc = (cue==side);
ma = mean(acc,3); % mean accuracy level
sa = std(acc,[],3)./sqrt(Nmc); % accuracy level: std
cols = 'br';
for ir=1:length(Rs)
    errorbar(ma(ir,:),sa(ir,:),'marker','.','color',cols(ir))
end
xlabel(ha,'diff level')
ylabel(ha,'accuracy (% correct)')
legend(ha,{'R=1','R=3'})
xtl = {'0:10','1:9','2:8','3:7','4:6','5:5'};
set(ha,'xlim',[0,length(ds)+1],'xtick',1:length(ds),'xticklabel',xtl)

% plot RTs
ha = subplot(3,2,4,'parent',hf,'nextplot','add');
mrt = mean(rt,3); % mean accuracy level
srt = std(rt,[],3)./sqrt(Nmc); % accuracy level: std
for ir=1:length(Rs)
    errorbar(mrt(ir,:),srt(ir,:),'marker','.','color',cols(ir))
end
xlabel(ha,'diff level')
ylabel(ha,'RT (a.u.)')
legend(ha,{'R=1','R=3'})
set(ha,'xlim',[0,length(ds)+1],'xtick',1:length(ds),'xticklabel',xtl)

% decision accuracy: regression analysis
idef = vec(cue); % when default preference aligns with cue
X = [zscore(vec(dif)),zscore(vec(rew)),zscore(idef)];
y = zscore(vec(acc));
% GLM_contrast(X,y,eye(3),'F',1)
CX = pinv(X'*X);
b = CX*X'*y;
vb = diag(var(y-X*b)*CX);
ha = subplot(3,2,5,'parent',hf,'nextplot','add');
plotUncertainTimeSeries(b,vb,[],ha)
set(ha,'xtick',1:3,'xticklabel',{'diff','rew','def'})
ylabel(ha,'partial correlation')

% RT: regression analysis
idef = vec(cue); % when default preference aligns with cue
X = [zscore(vec(dif)),zscore(vec(rew)),zscore(idef)];
y = zscore(vec(rt));
% GLM_contrast(X,y,eye(3),'F',1)
CX = pinv(X'*X);
b = CX*X'*y;
vb = diag(var(y-X*b)*CX);
ha = subplot(3,2,6,'parent',hf,'nextplot','add');
plotUncertainTimeSeries(b,vb,[],ha)
set(ha,'xtick',1:3,'xticklabel',{'diff','rew','def'})
ylabel(ha,'partial correlation')


%% decision accuracy vs response times
hf = figure('name','oMCD: decimotiv (2)');
ha = subplot(2,2,1,'parent',hf,'nextplot','add');
cols = 'brgmyc';
for id=1:length(ds)
    out = VBA_plotxy(vec(rt(:,id,:)),vec(acc(:,id,:)),5,ha,1);
    set(out.he,'color',cols(id),'markersize',18)%,'linewidth',1;
end
legend(ha,xtl)
xlabel(ha,'response time')
ylabel(ha,'accuracy')
% title(ha,'accruacy vs response time')

ha = subplot(2,2,2,'parent',hf,'nextplot','add');
cols = 'brgmyc';
for id=1:length(ds)
    out = VBA_plotxy(vec(rt(:,id,:)),vec(conf(:,id,:)),5,ha,1);
    set(out.he,'color',cols(id),'markersize',18)%,'linewidth',1;
end
legend(ha,xtl)
xlabel(ha,'response time')
ylabel(ha,'confidence')
% title(ha,'confidence vs response time')


ha = subplot(2,2,3,'parent',hf,'nextplot','add');
cols = 'br';
for ir=1:length(Rs)
    out = VBA_plotxy(vec(rt(ir,:,:)),vec(acc(ir,:,:)),5,ha,1);
    set(out.he,'color',cols(ir),'markersize',18)%,'linewidth',1;
end
legend(ha,{'R=1','R=2'})
xlabel(ha,'response time')
ylabel(ha,'accuracy')
% title(har,'accruacy vs response time')

ha = subplot(2,2,4,'parent',hf,'nextplot','add');
for ir=1:length(Rs)
    out = VBA_plotxy(vec(rt(ir,:,:)),vec(conf(ir,:,:)),5,ha,1);
    set(out.he,'color',cols(ir),'markersize',18)%,'linewidth',1;
end
legend(ha,{'R=1','R=2'})
xlabel(ha,'response time')
ylabel(ha,'confidence')
% title(ha,'confidence vs response time')



