% This script evaluates the impact of decision difficulty on the
% relationship between response time and confidence.
% This reproduces Figure 6 of Benon et al. (2024).

clear all
close all
warning off

%% Set MCD parameters
% fixed decision parameters
nu = 4; % effort cost power
R = 4; % decision importance
gamma = 4; % effort efficiency type #2
alpha = 4; % unitary cost of effort
v0 = 4; % initial (pre-decision) value variance
m0 = [0;0]; % initial value modes
maxz = 1; % max amount of resources
T = 1e2; % temporal horizon
kappa = maxz/T; % effort intensity
ts = 1:T; % temporal grid

%% run Monte-Carlo simulations
betas = [2:2:8];
N = length(betas);
Nmc = 5e2;

z0 = NaN(N,1);
Pc0 = NaN(N,1);
z = NaN(N,Nmc);
Pc = NaN(N,Nmc);

for i=1:N % run through simulations

    % set decision difficulty
    beta = betas(i); % effort efficiency type #1
    
    % proactive MCD: resource investment and confidence
    EPc = getPc(kappa*ts,beta,v0,m0,gamma,0); % anticpated confidence
    EQ0 = R*EPc - alpha*(kappa*ts).^nu; % anticipated discounted benefits
    tmcd = find(EQ0==max(EQ0),1); % index of max E[discounted benefit]
    Pc0(i) = EPc(tmcd); % E[confidence]
    z0(i) = kappa*tmcd; % effort investment
    
    % MDP threshold process
    [w,wP] = get_wMDP0(R,alpha,beta,gamma,kappa,nu,v0);
    
    % simulate dynamics of value representations
    Pct = NaN(Nmc,T);
    Q0t = NaN(Nmc,T);
    Pomcd = NaN(Nmc,1); % oMCD: confidence at response time
    tomcd = NaN(Nmc,1); % oMCD: response time
    for imc=1:Nmc % loop over Monte-Carlo simulations
        v = NaN(1,T);
        m = NaN(2,T);
        v(1) = 1./((1/v0)+beta*kappa);
        m(:,1) = m0 + sqrt(gamma*kappa)*randn(2,1);
        Pct(imc,1) = getPc(0,0,v(1),m(:,1),0,0);
        Q0t(imc,1) = R*Pct(imc,1) - alpha*kappa.^nu;
        for t=2:T % loop over time samples
            v(t) = 1./((1/v(t-1))+beta*kappa); % update variance
            m(:,t) = m(:,t-1) + sqrt(gamma*kappa)*randn(2,1); % update modes
            Pct(imc,t) = getPc(0,0,v(t),m(:,t),0,0); % evaluate current confidence
            Q0t(imc,t) = R*Pct(imc,t) - alpha*(kappa*t).^nu; % discounted benfit
        end
        % Detect oMCD decision properties
        tomcd(imc) = find(Pct(imc,:)>=wP,1); % oMCD: response time
        Pomcd(imc) = wP(tomcd(imc)); % oMCD: confidence
    end
    z(i,:) = kappa*tomcd';
    Pc(i,:) = Pomcd';
    
end


%% Display results
hf = figure('name','oMCD: effect of type #1 effort efficacy'); % Figure 6
ha = subplot(2,2,1,'parent',hf,'nextplot','add');
for i=1:N
    mz = mean(z(i,:));
    sz = std(z(i,:));
    errorbar(ha,betas(i),mz,sz,'color','k','marker','.','markersize',18);%,'linewidth',1);
    plot(ha,betas(i),z0(i),'color','r','marker','diamond')
end
xlabel(ha,'type #1 effort efficacy')
ylabel(ha,'resource investment')
set(ha,'xtick',betas)
title(ha,'A: resource investment')
set(ha,'xlim',[0,max(betas)+1])

ha = subplot(2,2,2,'parent',hf,'nextplot','add');
for i=1:N
    mp = mean(Pc(i,:));
    sp = std(Pc(i,:));
    errorbar(ha,betas(i),mp,sp,'color','k','marker','.','markersize',18);%,'linewidth',1)
    plot(ha,betas(i),Pc0(i),'color','r','marker','diamond')
end
xlabel(ha,'type #1 effort efficacy')
ylabel(ha,'confidence')
set(ha,'xtick',betas)
title(ha,'B: confidence')
set(ha,'xlim',[0,max(betas)+1])

ha = subplot(2,2,3,'parent',hf,'nextplot','add');
cols = 'brgm';
for i=1:N
    str{i} = ['\beta =',num2str(betas(i))];
    out = VBA_plotxy(z(i,:)',Pc(i,:)',10,ha,0);
    set(out.he,'color',cols(i),'markersize',18)%,'linewidth',1;
end
legend(ha,str)
xlabel(ha,'resource investment')
ylabel(ha,'achieved confidence')
title(ha,'C: effort/confidence relationship')
set(ha,'xlim',[0,1],'ylim',[0.5,1])

ha = subplot(2,2,4,'parent',hf,'nextplot','add');
for i=1:N
    [wMDP0,wp0] = get_wMDP0(R,alpha,betas(i),gamma,kappa,nu,v0);
    plot(ha,kappa*ts,wp0,'color',cols(i))
end
for i=1:N
    EPc = getPc(kappa*ts, betas(i),v0,m0,gamma,0); % anticipated confidence
    plot(ha,kappa*ts,EPc,'color',cols(i),'linestyle','--')
    EQ0 = R*EPc - alpha*(kappa*ts).^nu; % anticipated discounted benefits
    tau = find(EQ0==max(EQ0),1); % index of max E[discounted benefit]
    plot(ha,kappa*tau,EPc(tau),'marker','.','markersize',18,'color',cols(i))
end
set(ha,'xlim',[0,1],'ylim',[0.5,1])
legend(ha,str)
xlabel(ha,'resource')
ylabel(ha,'confidence')
title(ha,'D: optimal policy')



