% This script evaluates the impact of effort cost parameters on the
% dynamics of oMCD's optimal confidence threshold
% This reproduces Figure 3 of Benon et al. (2024).

clear all
close all
warning off

%% set default oMCD parameters
nu = 1; % effort cost power
R = 1; % decision importance
beta = 1;% effort efficiency type #1
gamma = 1; % effort efficiency type #2
alpha = 1; % unitary cost of effort
v0 = 1; % initial (pre-decision) value variance
m0 = [0;0]; % initial value modes
maxz = 1; % max amount of resources
T = 1e2; % nb of time samples
kappa = maxz/T; % rate of resource expanditure per unit of time
zs = kappa:kappa:1; % admissible resource amounts


%% effect of beta (decision difficulty)
hf = figure('name','oMCD: effect of parameters of oMCD''s optimal confidence threshold');
ha = subplot(2,2,1,'parent',hf,'nextplot','add');
title(ha,'A: type #1 effort efficacy')
betas = 2.^[-1:2];
cols = 'brgm';
for i=1:length(betas)
    beta = betas(i);
    str{i} = ['\beta =',num2str(beta)];
    % proactive MCD: expected confidence
    EPc = getPc(zs,beta,v0,m0,gamma,0); % anticpated confidence
    % oMCD's optimal confidence threshold
    [wMDP0,wp0] = get_wMDP0(R,alpha,beta,gamma,kappa,nu,v0);
    plot(ha,zs,wp0,'color',cols(i));%,'marker','.')
end
xlabel(ha,'resource')
ylabel(ha,'confidence threshold')
legend(ha,str)
set(ha,'ylim',[0.4,1])

%% effect of gamma
ha = subplot(2,2,2,'parent',hf,'nextplot','add');
title(ha,'B: type #2 effort efficacy')
beta = 1;
gammas = 2.^[-1:2];
cols = 'brgm';
for i=1:length(gammas)
    gamma = gammas(i);
    str{i} = ['\gamma =',num2str(gamma)];
    % proactive MCD: expected confidence
    EPc = getPc(zs,beta,v0,m0,gamma,0); % anticpated confidence
    % oMCD's optimal confidence threshold
    [wMDP0,wp0] = get_wMDP0(R,alpha,beta,gamma,kappa,nu,v0);
    plot(ha,zs,wp0,'color',cols(i));%,'marker','.')
end
xlabel(ha,'resource')
ylabel(ha,'confidence threshold')
legend(ha,str)
set(ha,'ylim',[0.4,1])

%% effect of alpha
ha = subplot(2,2,3,'parent',hf,'nextplot','add');
title(ha,'C: unitary effort cost')
gamma = 1;
alphas = 2.^[-1:2];
cols = 'brgm';
for i=1:length(alphas)
    alpha = alphas(i);
    str{i} = ['\alpha =',num2str(alpha)];
    % proactive MCD: expected confidence
    EPc = getPc(zs,beta,v0,m0,gamma,0); % anticpated confidence
    % oMCD's optimal confidence threshold
    [wMDP0,wp0] = get_wMDP0(R,alpha,beta,gamma,kappa,nu,v0);
    plot(ha,zs,wp0,'color',cols(i));%,'marker','.')
end
xlabel(ha,'resource')
ylabel(ha,'confidence threshold')
legend(ha,str)
set(ha,'ylim',[0.4,1])

%% effect of cost power
ha = subplot(2,2,4,'parent',hf,'nextplot','add');
title(ha,'D: effort cost power')
alpha = 1;
nus = 2.^[-1:2];
cols = 'brgm';
% pows = [2,2^-4];
% cols = 'br';
for i=1:length(nus)
    nu = nus(i);
    str{i} = ['\nu =',num2str(nu)];
    % proactive MCD: expected confidence
    EPc = getPc(zs,beta,v0,m0,gamma,0); % anticpated confidence
    % oMCD's optimal confidence threshold
    [wMDP0,wp0] = get_wMDP0(R,alpha,beta,gamma,kappa,nu,v0);
    plot(ha,zs,wp0,'color',cols(i));%,'marker','.')
end
xlabel(ha,'resource')
ylabel(ha,'confidence threshold')
legend(ha,str)
% legend(ha,{'superlinear cost','sublinear cost'})
set(ha,'ylim',[0.4,1])

