% This script exemplifies oMCD's control policy
% This reproduces Figures 2 and 4 of Benon et al. (2024).


clear all
close all
warning off


%% set MCD parameters
nu = 0.5; % effort cost power
R = 1; % decision importance
beta = 1; % effort efficiency type #1
gamma = 4; % effort efficiency type #2
alpha = 0.2; % unitary cost of effort
v0 = 1; % initial (pre-decision) value variance
m0 = [0;0]; % initial value modes
maxz = 1; % max amount of resources
T = 1e2; % nb of time samples
Kappa = maxz/T; % rate of resource expanditure per unit of time
zs = Kappa:Kappa:1; % admissible resource amounts
Nmc = 1e3; % nb of Monte-Carlo simulations

%% proactive MCD: get prior EVC anticipation
EPc = getPc(zs,beta,v0,m0,gamma,0); % anticipated confidence
EQ0 = R*EPc - alpha*zs.^nu; % anticipated discounted benefits
iz_opt = find(EQ0==max(EQ0),1); % best expected discounted benefit
z0_opt = zs(iz_opt); % resource investment
Pc0_opt = EPc(iz_opt); % expected confidence
EQ0_opt = EQ0(iz_opt); % expected discounted benefit

%% MDP threshold process
[wMDP0,wp0,wdmu,Pt,dmut] = get_wMDP0(R,alpha,beta,gamma,Kappa,nu,v0);

%% simulate dynamics of value representations
Pct = NaN(Nmc,T);
Q0t = NaN(Nmc,T);
P_mcd = NaN(Nmc,1);
t_mcd = NaN(Nmc,1);
Qpi_mcd = NaN(Nmc,1);
P_omcd = NaN(Nmc,1); % oMCD: confidence at response time
t_omcd = NaN(Nmc,1); % oMCD: response time
Qpi_omcd = NaN(Nmc,1); % oMCD: discunted benefit at response time
for imc=1:Nmc % loop over Monte-Carlo simulations
    v = NaN(1,T);
    m = NaN(2,T);
    v(1) = 1./((1/v0)+beta*Kappa);
    m(:,1) = m0 + sqrt(gamma*Kappa)*randn(2,1);
    Pct(imc,1) = getPc(0,0,v(1),m(:,1),0,0);
    Q0t(imc,1) = R*Pct(imc,1) - alpha*Kappa.^nu;
    for t=2:T % loop over time samples
        v(t) = 1./((1/v(t-1))+beta*Kappa); % update variance
        m(:,t) = m(:,t-1) + sqrt(gamma*Kappa)*randn(2,1); % update modes
        Pct(imc,t) = getPc(0,0,v(t),m(:,t),0,0); % evaluate current confidence
        Q0t(imc,t) = R*Pct(imc,t) - alpha*(Kappa*t).^nu; % discounted benfit
    end
    % detect proactive MCD decision
    t_mcd(imc) = round(z0_opt./Kappa); % MCD: stop when predefined time is reached
    P_mcd(imc) = Pct(imc,t_mcd(imc,1)); % MCD: achieved confidence level
    Qpi_mcd(imc) = Q0t(imc,t_mcd(imc)); % MCD: discounted benefit
    % Detect oMCD decision properties
    t_omcd(imc) = find(Pct(imc,:)>=wp0,1); % oMCD: response time
    P_omcd(imc) = Pct(imc,t_omcd(imc)); % oMCD: confidence
    Qpi_omcd(imc) = Q0t(imc,t_omcd(imc)); % oMCD: discounted benefit
end

%% oracle (reference point)
[Qpi_oracle,t_oracle] = max(Q0t,[],2);
P_oracle = NaN(Nmc,1);
for imc=1:Nmc
    P_oracle(imc) = Pct(imc,t_oracle(imc)); % oracle: store confidence achieved
end

%% Display oMCD optimal stopping strategy
hf = figure('name',['oMCD optimal stopping strategy (cost=alpha*t^',num2str(nu),')']); % Figure 2
% plot dynamics of discounted benefits
ha2 = subplot(2,2,1,'parent',hf,'nextplot','add');
me = mean(Q0t);
e05 = prctile(Q0t,5,1);
e95 = prctile(Q0t,95,1);
mtmp = mean([e05;e95],1);
vtmp = (mtmp-e95).^2;
[~,~,hp] = plotUncertainTimeSeries(mtmp,vtmp,[],ha2,1,'b');
delete(hp)
plot(ha2,me,'b')
plot(ha2,(z0_opt/Kappa)*[1,1],get(ha2,'ylim'),'r')
plot(ha2,get(ha2,'xlim'),EQ0_opt*[1,1],'r')
plot(ha2,wMDP0,'k--')
xlabel(ha2,'time (t)')
ylabel(ha2,'Q(0,\Delta\mu(t))')
title(ha2,'A: net benefit')
% plot confidence and thresholds
ha = subplot(2,2,2,'parent',hf,'nextplot','add');
mp = mean(Pct); % average confidence at time t
plot(ha,mp,'b')
plot(ha,wp0,'k--')
e05 = prctile(Pct,5,1);
e95 = prctile(Pct,95,1);
mtmp = mean([e05;e95],1);
vtmp = (mtmp-e95).^2;
[~,~,hp] = plotUncertainTimeSeries(mtmp,vtmp,[],ha,1,'b');
delete(hp)
plot(ha,[1,1]*z0_opt/Kappa,get(ha,'ylim'),'r')
plot(ha,get(ha,'xlim'),Pc0_opt*[1,1],'r')
xlabel(ha,'time (t)')
ylabel(ha,'P_c(\Delta\mu(t))')
title(ha,'B: confidence')
% legend(ha,{'current confidence','oMCD threshold'})


%% Display confidence, resources and discounted benefits of MCD strategies
hf = figure('name',['oMCD vs prospective MCD (cost=alpha*t^',num2str(nu),')']); % Figure 4
xstr = {'oMCD','MCD','oracle'};
% plot invested resources (per strategy)
ha = subplot(2,2,1,'parent',hf,'nextplot','add');
mz = nanmean(Kappa*t_omcd);
sz = nanstd(Kappa*t_omcd);
bar(1,mz,'parent',ha,'facecolor','k')
errorbar(1,mz,sz,'parent',ha,'color','k')
mz = nanmean(Kappa*t_mcd);
sz = nanstd(Kappa*t_mcd);
bar(2,mz,'parent',ha,'facecolor','r')
errorbar(2,mz,sz,'parent',ha,'color','r')
mz = nanmean(Kappa*t_oracle);
sz = nanstd(Kappa*t_oracle);
bar(3,mz,'parent',ha,'facecolor','g')
errorbar(3,mz,sz,'parent',ha,'color','g')
set(ha,'xtick',1:3,'xticklabel',xstr)
title(ha,'A: resource investment')
% xlabel(ha,'thresholding strategy')
% ylabel(ha,'\kappa\times\tau')
% plot confidence achieved (per strategy)
ha = subplot(2,2,2,'parent',hf,'nextplot','add');
mp = nanmean(P_omcd);
sp = nanstd(P_omcd);
bar(1,mp,'parent',ha,'facecolor','k')
errorbar(1,mp,sp,'parent',ha,'color','k')
mp = nanmean(P_mcd);
sp = nanstd(P_mcd);
bar(2,mp,'parent',ha,'facecolor','r')
errorbar(2,mp,sp,'parent',ha,'color','r')
mp = nanmean(P_oracle);
sp = nanstd(P_oracle);
bar(3,mp,'parent',ha,'facecolor','g')
errorbar(3,mp,sp,'parent',ha,'color','g')
set(ha,'xtick',1:3,'xticklabel',xstr,'ytick',[0.5,0.75,1],'ylim',[0.5,1.1])
title(ha,'B: achieved confidence')
% xlabel(ha,'thresholding strategy')
% ylabel(ha,'P_c(\Delta\mu(\tau))')
% plot EVC (per strategy)
ha = subplot(2,2,3,'parent',hf,'nextplot','add');
me = nanmean(Qpi_omcd);
se = nanstd(Qpi_omcd);
bar(1,me,'parent',ha,'facecolor','k')
errorbar(1,me,se,'parent',ha,'color','k')
me = nanmean(Qpi_mcd);
se = nanstd(Qpi_mcd);
bar(2,me,'parent',ha,'facecolor','r')
errorbar(2,me,se,'parent',ha,'color','r')
me = nanmean(Qpi_oracle);
se = nanstd(Qpi_oracle);
bar(3,me,'parent',ha,'facecolor','g')
errorbar(3,me,se,'parent',ha,'color','g')
set(ha,'xtick',1:3,'xticklabel',cat(2,xstr,'oracle'))
title(ha,'C: net benefit')
% xlabel(ha,'thresholding strategy')
% ylabel(ha,'Q(0,\Delta\mu(\tau))')
% plot achieved confidence against decision time
ha = subplot(2,2,4,'parent',hf,'nextplot','add');
out = VBA_plotxy(Kappa*t_omcd,P_omcd,10,ha,0);
set(out.he,'color','k','linewidth',1,'markersize',18);
errorbar(ha,mean(Kappa*t_mcd),mean(P_mcd),std(P_mcd),'color','r','linewidth',1,'marker','.','markersize',18)
out = VBA_plotxy(Kappa*t_oracle,P_oracle,10,ha,0);
set(out.he,'color','g','linewidth',1,'markersize',18);
plot(ha,zs,wp0,'k--')
set(ha,'ytick',[0.5,0.75,1],'ylim',[0.5,1.1])
legend(ha,{'oMCD','MCD','oracle'})
xlabel(ha,'resource investment')
ylabel(ha,'achieved confidence')
title(ha,'D: effort/confidence relationship')
