% This demo tests the optimality of oMCD's policy, under two different
% decision processes (namely: Bayesian value denoising and progressive
% attribute integration).
% This reproduces Figures 7 and 8 of Benon et al. (2024).

clear all
close all
warning off

%% Bayesian value denoising: validate analytical derivation of E[d(t)^2]

N = 1e3; % nb of random samples to derive expectations
Nmc = 1e2; % nb of Monte-Carlo simulations (=nb of param set samples)
Ed = NaN(Nmc,2); % E[delta]
Ed2 = NaN(Nmc,2); % E[delta^2]
for imc=1:Nmc % loop over Monte-Carlo simulations
    % sample Bayesian agent parameters
    Sigma = 1+abs(randn);
    mu0 = randn;
    sigma0 = abs(randn);
    % sample hidden values
    V = mu0 + sqrt(sigma0)*randn(N,1);
    % sample noisy value signals
    y = repmat(V,1,N) + sqrt(Sigma)*randn(N,N);
    % Bayesian update
    [mu,sigma,lambda] = myBayesianUpdate(mu0,sigma0,Sigma,y);
    delta = mu-mu0;
    % Derive empirical and theoretical estimates of E[delta] and E[delta^2]
    Ed(imc,1) = mean(delta(:)); % empirical E[delta]
    Ed(imc,2) = 0; % theoretical E[delta]
    Ed2(imc,1) = mean(delta(:).^2); % empirical E[delta^2]
    Ed2(imc,2) = (lambda.^2)*(Sigma+sigma0); % theoretical E[delta^2]
  
end
hf = figure('name','Validating Equation A2');
ha = subplot(2,2,1,'parent',hf,'nextplot','add');
plot(ha,Ed(:,2),Ed(:,1),'k.')
xlabel(ha,'theoretical')
ylabel(ha,'Monte-Carlo estimate')
title(ha,'E[\delta(t)|\mu(t)]')
ha = subplot(2,2,2,'parent',hf,'nextplot','add');
plot(ha,Ed2(:,2),Ed2(:,1),'k.')
plot(ha,get(ha,'xlim'),get(ha,'xlim'),'r--')
xlabel(ha,'Equation A3')
ylabel(ha,'Monte-Carlo estimate')
title(ha,'E[\delta^2|\mu(t)]')



%% Bayesian value denoising: oMCD versus ideal policy

% sample decision parameters
Sigma = 2;
v0 = 1;
% sample MCD parameters
R = 1; % decision importance
kappa = 0.05; % effort intensity
alpha = 1; % effort sensitivity
nu = 2; % effort cost power
% appproximate effort efficacy parameters
T = 1/kappa; % temporal horizon
ts = 0:T; % temporal grid
beta = 1/(kappa.*Sigma); % type #1 effort efficacy
vt = 1./((1/v0)+(ts/Sigma)); % value variance
Ed2t = (Sigma+vt)./(((Sigma./vt)+1).^2); % E[delta^2]
gamma = mean(Ed2t/kappa); % type #2 effort efficacy
% derive oMCD threshold dynamics
[w,wP] = get_wMDP0(R,alpha,beta,gamma,kappa,nu,v0);
[w0,wP0] = get_wMDP_BayesianValueDenoising(R,alpha,Sigma,kappa,nu,v0);
N = 1e3; % nb of sample path trajectories
Z = NaN(2,N);
Pc = NaN(2,N);
Q = NaN(2,N);
for i=1:N % loop over sample path trajectories
    mut = [zeros(2,1),NaN(2,T)]; % value mode difference
    a = ones(2,T+1); % stop/continue actions
    V = sqrt(v0)*randn(2,1); % hidden value
    y = repmat(V,1,T) + sqrt(Sigma)*randn(2,T); % noisy value signal
    for t=1:T
        % Bayesian value denoising
        [mut(:,t+1)] = myBayesianUpdate(mut(:,t),vt(t),Sigma,y(:,t));
        % oMCD policy: stop or continue
        dmut = mut(1,t+1)-mut(2,t+1);
        Pc_now = getPc_dmu(0,0,vt(t+1),dmut,0,0); % current confidence
        a(1,t+1) = (Pc_now<=wP(t)); % 0=stop, 1=continue
        if (prod(a(1,1:t))==1) && (a(1,t+1)==0) % 1st stop
            Z(1,i) = kappa*t;
            Pc(1,i) = Pc_now;
            Q(1,i) = R*Pc_now - alpha*(kappa*t).^nu;
        end
        % ideal policy: stop or continue
        a(2,t+1) = (Pc_now<=wP0(t)); % 0=stop, 1=continue
        if (prod(a(2,1:t)))==1 && (a(2,t+1)==0) % 1st stop
            Z(2,i) = kappa*t;
            Pc(2,i) = Pc_now;
            Q(2,i) = R*Pc_now - alpha*(kappa*t).^nu;
        end
    end % end temporal grid
    
end % end sample path trajectories


hf = figure; % Figure A1
ha = subplot(2,2,1,'parent',hf,'nextplot','add');
plot(ha,ts,Ed2t,'r')
plot(ha,get(ha,'xlim'),kappa*gamma*[1,1],'b')
xlabel(ha,'time')
ylabel(ha,'E[\delta(t)^2]')
title(ha,'A: perturbation variance')
legend(ha,{'ideal policy','oMCD''s approximation'})
ha = subplot(2,2,2,'parent',hf,'nextplot','add');
plot(ha,wP0,'r')
plot(ha,wP,'b')
xlabel(ha,'time')
ylabel(ha,'\omega_P(t)')
legend(ha,{'ideal policy','oMCD'})
title(ha,'B: confidence threshold')
set(ha,'ylim',[0.5,1])
ha = subplot(2,2,3,'parent',hf,'nextplot','add');
zs = vec(Z(2,:));
[py,gridy] = empiricalHist(zs,1);
plot(ha,gridy,py/mean(diff(gridy)),'r');
zs = vec(Z(1,:));
[py,gridy] = empiricalHist(zs,1);
plot(ha,gridy,py/mean(diff(gridy)),'b');
legend(ha,{'ideal policy','oMCD'})
xlabel(ha,'invested z')
ylabel(ha,'p(z|policy)')
title(ha,'C: invested resources')
set(ha,'xlim',[0,1],'ytick',[])
ha = subplot(2,2,4,'parent',hf,'nextplot','add');
ps = vec(Pc(2,:));
[py,gridy] = empiricalHist(ps,1);
plot(ha,gridy,py/mean(diff(gridy)),'r');
ps = vec(Pc(1,:));
[py,gridy] = empiricalHist(ps,1);
plot(ha,gridy,py/mean(diff(gridy)),'b');
legend(ha,{'ideal policy','oMCD'})
xlabel(ha,'Pc(z)')
ylabel(ha,'p(Pc(z)|policy)')
title(ha,'D: achieved confidence')
set(ha,'xlim',[0.5,1],'ytick',[])



%% Bayesian value denoising: test oMCD stopping policy
N = 5e2; % nb of random samples to derive expectations
Nmc = 2e2; % nb of Monte-Carlo simulations (=nb of param set samples)
Z = NaN(Nmc,2,N); % resource investments
Pc = NaN(Nmc,2,N); % achieved confidence
Q = NaN(Nmc,2,N); % net benefits
Pmcd = NaN(4,Nmc); % MCD parameters
R = 1; % useless (since we sample over alpha below)
kappa = 0.05; % effort intensity
T = 1/kappa; % temporal horizon
ts = 0:T; % temporal grid
for imc=1:Nmc % loop over Monte-Carlo simulations
    imc
    % sample Bayesian agent parameters
    Sigma = abs(randn);
    v0 = abs(randn);
    % sample MCD parameters
    alpha = abs(randn);
    nu = abs(randn); % effort cost power
    % appproximate effort efficacy parameters
    beta = 1/(kappa.*Sigma); % type #1 effort efficacy
    vt = 1./((1/v0)+(ts/Sigma)); % value variance
    Ed2t = (Sigma+vt)./(((Sigma./vt)+1).^2); % E[delta^2]
    gamma = mean(Ed2t/kappa); % type #2 effort efficacy
    % derive oMCD threshold dynamics
    [w,wP] = get_wMDP0(R,alpha,beta,gamma,kappa,nu,v0);
    [w0,wP0] = get_wMDP_BayesianValueDenoising(R,alpha,Sigma,kappa,nu,v0);
    % simulate Bayesian value denoising
    for i=1:N % loop over sample path trajectories
        mut = [zeros(2,1),NaN(2,T)]; % value mode difference
        a = ones(2,T+1); % stop/continue actions
        V = sqrt(v0)*randn(2,1); % hidden value
        y = repmat(V,1,T) + sqrt(Sigma)*randn(2,T); % noisy value signal
        for t=1:T 
            % Bayesian value denoising
            [mut(:,t+1)] = myBayesianUpdate(mut(:,t),vt(t),Sigma,y(:,t));
            % oMCD policy: stop or continue
            dmut = mut(1,t+1)-mut(2,t+1);
            Pc_now = getPc_dmu(0,0,vt(t+1),dmut,0,0); % current confidence
            a(1,t+1) = (Pc_now<=wP(t)); % 0=stop, 1=continue
            if (prod(a(1,1:t))==1) && (a(1,t+1)==0) % 1st stop
                Z(imc,1,i) = kappa*t;
                Pc(imc,1,i) = Pc_now;
                Q(imc,1,i) = R*Pc_now - alpha*(kappa*t).^nu;
            end
            % ideal policy: stop or continue
            a(2,t+1) = (Pc_now<=wP0(t)); % 0=stop, 1=continue
            if (prod(a(2,1:t)))==1 && (a(2,t+1)==0) % 1st stop
                Z(imc,2,i) = kappa*t;
                Pc(imc,2,i) = Pc_now;
                Q(imc,2,i) = R*Pc_now - alpha*(kappa*t).^nu;
            end
        end % end temporal grid
        if VBA_isweird(Z(imc,:,i))
            figure
            plot(wP)
            hold on
            plot(wP0,'r')
            plot(mut','k')
            keyboard
        end
    end % end sample path trajectories
    
    % store MCD parameters 
    Pmcd(:,imc) = [alpha;beta;gamma;nu];
    
end

gammas = Pmcd(3,:)';
x = sig(3*(gammas-mean(gammas)));
cols = [x,zeros(Nmc,1),1-x];

hf = figure('name','Bayesian value denoising'); % Figure 7
titles = {'A: resource investment','B: confidence','C: net benefit'};
for i=1:3
    ha(i) = subplot(2,2,i,'parent',hf,'nextplot','add');
    title(ha(i),titles{i})
end
for imc=1:Nmc
    plot(ha(1),mean(Z(imc,2,:),3),mean(Z(imc,1,:),3),'.','color',cols(imc,:))
    plot(ha(2),mean(Pc(imc,2,:),3),mean(Pc(imc,1,:),3),'.','color',cols(imc,:))
    plot(ha(3),mean(Q(imc,2,:),3),mean(Q(imc,1,:),3),'.','color',cols(imc,:))
end
set(ha(1),'xlim',[0,1],'ylim',[0,1]);
set(ha(2),'xlim',[0.4,1],'ylim',[0.4,1]);
for i=1:3
    plot(ha(i),get(ha(i),'xlim'),get(ha(i),'xlim'),'k--')
    xlabel(ha(i),'ideal policy')
    ylabel(ha(i),'oMCD policy')
end






%% Attribute integration: eyeball ideal policy

% sample decision parameters
R = 1;
alpha = 1;
nu = 4;
k = 8; % nb of attributes
eta = 1*randn(k,1);
zeta = (randn(k,1)).^2;
W = randn(k,1);
Nmc = 1e4;

[pit,pi0,gridmu,gridv,Pdmu,Pv] = get_pi_AttributeIntegration(R,alpha,nu,eta,zeta,W);

v0 = (W.^2)'*zeta; % prior variance over option values
kappa = 1/k; % effort intensity
T = k; % temporal horizon
idmup0 = find(gridmu>=0);
mu0 = W'*eta; % prior mean over option values
nmu = length(gridmu);
nv = length(gridv);

hf = figure('name','Attribute integration: transition densities');
ha = subplot(2,2,1,'parent',hf);
imagesc(Pdmu,'parent',ha)
set(ha,'xdir','normal','ydir','normal')
axis(ha,'tight','square')
title(ha,'A: P(\Delta\mu(t)|\Delta\mu(t-1))')
xlabel(ha,'\Delta\mu(t)')
ylabel(ha,'\Delta\mu(t-1)')
set(ha,'xtick',[],'ytick',[])
ha = subplot(2,2,2,'parent',hf);
imagesc(Pv,'parent',ha)
set(ha,'xdir','normal','ydir','normal')
axis(ha,'tight','square')
title(ha,'B: P(\sigma(t)|\sigma(t-1))')
xlabel(ha,'\sigma(t)')
ylabel(ha,'\sigma(t-1)')
set(ha,'xtick',[],'ytick',[])

% sample dmu(t) and v(t) trajectories
vt = [v0*ones(Nmc,1),NaN(Nmc,T)];
dmut = [zeros(Nmc,1),NaN(Nmc,T)];
dvt = NaN(Nmc,T);
divt = NaN(Nmc,T);
Pvt = zeros(nv,T);
Pmut = zeros(nmu,T);
for imc=1:Nmc
    kt = randperm(k);
    for t=1:T % loop over time
        % value representation update
        xt = eta(kt(t)) + sqrt(zeta(kt(t)))*randn(2,1); % sample attributes
        dt = W(kt(t))*(xt-eta(kt(t))); % change in value mode
        dmut(imc,t+1) = dmut(imc,t) + (dt(1)-dt(2)); % value mode update
        vt(imc,t+1) = vt(imc,t) - (W(kt(t)).^2)*zeta(kt(t)); % variance update
        % change in variance and precision
        dvt(imc,t) = vt(imc,t) - vt(imc,t+1);
        divt(imc,t) = 1./vt(imc,t) - 1./vt(imc,t+1);
        % fill in distribution of dmu(t)
        d1 = abs(dmut(imc,t+1)-gridmu);
        i1 = find(d1==min(d1));
        Pmut(i1,t) = Pmut(i1,t)+1;
        % fill in distribution of v(t)
        d1 = abs(vt(imc,t+1)-gridv);
        i1 = find(d1==min(d1));
        Pvt(i1,t) = Pvt(i1,t)+1;
    end
end
for t=1:T
    Pmut(:,t) = Pmut(:,t)./sum(Pmut(:,t));
    Pvt(:,t) = Pvt(:,t)./sum(Pvt(:,t));
end
ha = subplot(2,2,3,'parent',hf,'nextplot','add');
imagesc(Pmut,'parent',ha)
set(ha,'xdir','normal','ydir','normal')
axis(ha,'tight','square')
title(ha,'C: P(\Delta\mu(t))')
xlabel(ha,'decision time (t)')
ylabel(ha,'\Delta\mu(t)')
set(ha,'xtick',[],'ytick',[])

ha = subplot(2,2,4,'parent',hf,'nextplot','add');
imagesc(Pvt,'parent',ha)
set(ha,'xdir','normal','ydir','normal')
axis(ha,'tight','square')
title(ha,'D: P(\sigma(t))')
xlabel(ha,'decision time (t)')
ylabel(ha,'\sigma(t)')
set(ha,'xtick',[],'ytick',[])


% eyeball ideal stopping policy
hf0 = figure('name','Attribute integration: control policies'); % Figure A2
ha0 = subplot(2,2,1,'parent',hf0,'nextplot','add');
% At t=0
contour(pi0,1,'parent',ha0,'color',[0,0,0]) % display optimal frontier
legstr{1} = ['t=0'];
% Loop over time
for t=1:(T-1)
    contour(pit(:,:,t),1,'parent',ha0,'color',[(T-t-1)/(T-1),0,t/(T-1)]) % display optimal frontier
    legstr{t+1} = ['t=',num2str(t)];
end
title(ha0,'A: ideal policy frontier')
xlabel(ha0,'\sigma(t)')
ylabel(ha0,'\Delta\mu(t)')
set(ha0,'xtick',[],'ytick',[])
legend(ha0,legstr)


% eyeball induced decisions
a = NaN(Nmc,T);
indmu = NaN(Nmc,T);
indv = NaN(Nmc,T);
tau = NaN(Nmc,1);
Pc_tau = NaN(Nmc,1);
choice = NaN(Nmc,1);
DV = dmut(:,T+1); % option values (full attribute integration)
ha = subplot(2,2,2,'parent',hf0,'nextplot','add');
for imc=1:Nmc
    for t=1:T % loop over time
        % stop or continue
        dmu = abs(gridmu-dmut(imc,t+1));
        indmu(imc,t) = find(dmu==min(dmu));
        dv = abs(gridv-vt(imc,t+1));
        indv(imc,t) = find(dv==min(dv));
        a(imc,t) = pit(indmu(imc,t),indv(imc,t),t);
    end
    % find decision time
    tmp = find(a(imc,:)==0,1);
    if ~isempty(tmp)
        tau(imc) = tmp;
    else
        tau(imc) = T;
    end
    % decision type
    tmp = sign(dmut(imc,tau(imc)+1));
    choice(imc) = isequal(tmp,sign(DV(imc)));
    % evaluate confidence
    Pc_tau(imc) = getPc_dmu(0,0,vt(imc,tau(imc)+1),dmut(imc,tau(imc)+1),0,0);
    % eyeball trajectory
    if imc<64
        plot(ha,indv(imc,:),indmu(imc,:),'color',0.8*[1,1,1])
        plot(ha,indv(imc,tau(imc)),indmu(imc,tau(imc)),'k.')
%         pause
    end
end
set(ha,'xlim',get(ha0,'xlim'),'ylim',get(ha0,'ylim'))
title(ha,'B: stochastic trajectories')
xlabel(ha,'\sigma(t)')
ylabel(ha,'\Delta\mu(t)')
set(ha,'xtick',[],'ytick',[])

% Simulate oMCD decision times and confidence
gamma = v0;
ts = (1:k+1)'-1;


Evt0 = v0*(k-ts)./k; % E[v(t)|v0]
tmp = mean((W.^4).*zeta.^2)-(v0/k)^2;
Vvt0 = (ts.*(k-ts)).*tmp/k; % V[v(t)|v0]
Edvt0 = v0/k;
Vdvt0 = mean((W.^4).*zeta.^2)-(v0/k)^2;
Eivt0 = 1./Evt0 - 1/v0;% + Vvt0./(Evt0.^3);

% beta = mean(vec(-divt(:,1:T-1)))./kappa;
Eivt = mean(cumsum(-divt,2),1)';
beta = mean(Eivt0(2:(T-1))./(kappa.*ts(2:(T-1))));
[w,wP,wdmu] = get_wMDP0(R,alpha,beta,gamma,kappa,nu,v0);
Pct = getPc_dmu(0,0,vt(:,2:(T+1)),dmut(:,2:(T+1)),0,0);
a0 = NaN(Nmc,T);
tau0 = NaN(Nmc,1);
Pc_tau0 = NaN(Nmc,1);
choice0 = NaN(Nmc,1);
for imc=1:Nmc
    a0(imc,:) = (Pct(imc,:)<wP); % 1: stop, 0: continue
    % find decision time
    tmp = find(a0(imc,:)==0,1);
    if ~isempty(tmp)
        tau0(imc) = tmp;
    else
        tau0(imc) = T;
    end
    % decision type
    tmp = sign(dmut(imc,tau0(imc)+1));
    choice0(imc) = isequal(tmp,sign(DV(imc)));
    % evaluate confidence
    Pc_tau0(imc) = Pct(imc,tau0(imc));
end

% Compare decision times
ha = subplot(2,2,3,'parent',hf0,'nextplot','add');
[ny,nx] = hist(tau,1:T);
plot(ha,kappa*nx,ny./sum(ny),'r')
[ny,nx] = hist(tau0,1:T);
plot(ha,kappa*nx,ny./sum(ny),'b')
set(ha,'ytick',[],'xlim',[0,1])
legend(ha,{'ideal policy','oMCD'})
xlabel(ha,'invested z')
ylabel(ha,'p(z|policy)')
title(ha,'C: resource investments')


% Compare confidence
ha = subplot(2,2,4,'parent',hf0,'nextplot','add');
[py,gridy] = empiricalHist(Pc_tau,.1);
dy = mean(diff(gridy));
plot(ha,gridy,py/dy,'r')
[py,gridy] = empiricalHist(Pc_tau0,.1);
dy = mean(diff(gridy));
plot(ha,gridy,py/dy,'b')
set(ha,'ytick',[],'xlim',[0.5,1])
legend(ha,{'ideal policy','oMCD'})
xlabel(ha,'Pc(z)')
ylabel(ha,'p(Pc(z)|policy)')
title(ha,'D: achieved confidence')




%% Attribute integration: how do decision features look like?
hf = figure;
% ideal policy: how do decision times look like?
xl = [min(DV),max(DV)];
ha = subplot(2,2,1,'parent',hf,'nextplot','add');
VBA_plotxy(DV,kappa*tau,8,ha)
medP = median(Pc_tau);
i1 = find(Pc_tau>medP);
out = VBA_plotxy(DV(i1),kappa*tau(i1),8,ha);
set(out.he,'color','b')
i1 = find(Pc_tau<medP);
out = VBA_plotxy(DV(i1),kappa*tau(i1),8,ha);
set(out.he,'color','r')
title(ha,'A: ideal policy')
xlabel(ha,'dv')
ylabel(ha,'invested resources')
legend(ha,{'all trials','high confidence','low confidence'})
set(ha,'xtick',[],'xlim',xl,'ylim',[0,1])

% ideal policy: how does confidence look like?
ha = subplot(2,2,2,'parent',hf,'nextplot','add');
VBA_plotxy(DV,Pc_tau,8,ha)
i1 = find(choice==1);
p1 = 100*length(i1)/Nmc;
str1 = ['value-consistent (',num2str(p1,3),'%)'];
out = VBA_plotxy(DV(i1),Pc_tau(i1),8,ha);
set(out.he,'color','b')
i1 = find(choice==0);
p0 = 100*length(i1)/Nmc;
str0 = ['value-inconsistent (',num2str(p0,3),'%)'];
out = VBA_plotxy(DV(i1),Pc_tau(i1),8,ha);
set(out.he,'color','r')
title(ha,'B: ideal policy')
xlabel(ha,'dv')
ylabel(ha,'confidence')
legend(ha,{'all trials',str1,str0})
set(ha,'xtick',[],'xlim',xl,'ylim',[0.4,1])

% oMCD: how do decision times look like?
ha = subplot(2,2,3,'parent',hf,'nextplot','add');
VBA_plotxy(DV,kappa*tau0,8,ha)
medP0 = median(Pc_tau0);
i1 = find(Pc_tau0>medP0);
out = VBA_plotxy(DV(i1),kappa*tau0(i1),8,ha);
set(out.he,'color','b')
i1 = find(Pc_tau0<medP0);
out = VBA_plotxy(DV(i1),kappa*tau0(i1),8,ha);
set(out.he,'color','r')
title(ha,'C: oMCD')
xlabel(ha,'dv')
ylabel(ha,'invested resources')
legend(ha,{'all trials','high confidence','low confidence'})
set(ha,'xtick',[],'xlim',xl,'ylim',[0,1])

% oMCD: how does confidence look like?
ha = subplot(2,2,4,'parent',hf,'nextplot','add');
VBA_plotxy(DV,Pc_tau0,8,ha)
i1 = find(choice0==1);
p1 = 100*length(i1)/Nmc;
str1 = ['value-consistent (',num2str(p1,3),'%)'];
out = VBA_plotxy(DV(i1),Pc_tau0(i1),8,ha);
set(out.he,'color','b')
i1 = find(choice0==0);
p0 = 100*length(i1)/Nmc;
str0 = ['value-inconsistent (',num2str(p0,3),'%)'];
out = VBA_plotxy(DV(i1),Pc_tau0(i1),8,ha);
set(out.he,'color','r')
title(ha,'D: oMCD')
xlabel(ha,'dv')
ylabel(ha,'confidence')
legend(ha,{'all trials',str1,str0})
set(ha,'xtick',[],'xlim',xl,'ylim',[0.4,1])


%% Attribute integration: test oMCD control policy
N = 5e2; % nb of random samples to derive expectations
Nmc = 2e2; % nb of Monte-Carlo simulations (=nb of param set samples)
Z = NaN(Nmc,2,N); % invested resources
Pc = NaN(Nmc,2,N); % achieved confidence
Q = NaN(Nmc,2,N); % net benefit
Pmcd = NaN(4,Nmc); % sampled MCD parameteters
R = 1; % useless (since we sample over alpha below)
k = 10; % nb of attributes
kappa = 1/k; % effort intensity
T = 1/kappa; % temporal horizon
ts = 1:T; % temporal grid

for imc=1:Nmc % loop over Monte-Carlo simulations
    imc
    % sample decision parameters
    eta = randn(k,1); % attributes: prior means
    zeta = (randn(k,1)).^2; % attributes: prior variance
    W = randn(k,1); % attributes: weights
    % sample MCD parameters
    alpha = abs(randn); % effort cost sensitivity
    nu = abs(randn); % effort cost power
    % derive ideal policy
    [pit,pi0,gridmu,gridv,Pdmu,Pv] = get_pi_AttributeIntegration(R,alpha,nu,eta,zeta,W);
    % derive oMCD policy
    v0 = (W.^2)'*zeta; % prior variance over option values
    gamma = v0; % type #2 effort efficacy
    Evt0 = v0*(k-ts)./k; % E[v(t)]
%     tmp = mean((W.^4).*zeta.^2)-(v0/k)^2;
%     Vvt0 = (ts.*(k-ts)).*tmp/k; % V[v(t)]
    Eivt0 = 1./Evt0 - 1/v0;% + Vvt0./(Evt0.^3); % E[1/v(t)-1/v0]
    beta = mean(Eivt0(2:(T-1))./(kappa.*ts(2:(T-1)))); % type #1 effort efficacy
    [w,wP,wdmu] = get_wMDP0(R,alpha,beta,gamma,kappa,nu,v0);
    % sample trajectories and obtain decision features
    for i=1:N % loop over sample path trajectories
        dmut = [0,NaN(1,T)]; % value mode difference
        vt = [v0,NaN(1,T)]; % value variance
        a = ones(2,T+1); % stop/continue actions
        kt = randperm(k); % random permutation of attribute indices
        for t=1:T % loop over time
            % value representation update
            xt = eta(kt(t)) + sqrt(zeta(kt(t)))*randn(2,1); % sample attributes
            deltat = W(kt(t))*(xt-eta(kt(t))); % change in value mode
            dmut(t+1) = dmut(t) + (deltat(1)-deltat(2)); % value mode update
            vt(t+1) = vt(t) - (W(kt(t)).^2)*zeta(kt(t)); % variance update
            % oMCD policy: stop or continue
            Pc_now = getPc_dmu(0,0,vt(t+1),dmut(t+1),0,0); % current confidence
            a(1,t+1) = (Pc_now<=wP(t)); % oMCD policy
            if (prod(a(1,1:t))==1) && (a(1,t+1)==0) % 1st stop
                Z(imc,1,i) = kappa*t;
                Pc(imc,1,i) = Pc_now;
                Q(imc,1,i) = R*Pc_now - alpha*(kappa*t).^nu;
            end
            % ideal policy: stop or continue
            a(2,t+1) = myPiStar(dmut,vt,pit,t,gridmu,gridv);
            if (prod(a(2,1:t)))==1 && (a(2,t+1)==0) % 1st stop
                Z(imc,2,i) = kappa*t;
                Pc(imc,2,i) = Pc_now;
                Q(imc,2,i) = R*Pc_now - alpha*(kappa*t).^nu;
            end
            
        end
        
    end
    
    Pmcd(:,imc) = [alpha;beta;gamma;nu];
    
end

betas = Pmcd(2,:)';
x = sig(3*(betas-mean(betas)));
cols = [x,zeros(Nmc,1),1-x];

hf = figure('name','Attribute integration'); % Figure 8
titles = {'A: resource investment','B: confidence','C: net benefit'};
for i=1:3
    ha(i) = subplot(2,2,i,'parent',hf,'nextplot','add');
    title(ha(i),titles{i})
end
for imc=1:Nmc
    plot(ha(1),mean(Z(imc,2,:),3),mean(Z(imc,1,:),3),'.','color',cols(imc,:))
    plot(ha(2),mean(Pc(imc,2,:),3),mean(Pc(imc,1,:),3),'.','color',cols(imc,:))
    plot(ha(3),mean(Q(imc,2,:),3),mean(Q(imc,1,:),3),'.','color',cols(imc,:))
end
set(ha(1),'xlim',[0,1],'ylim',[0,1]);
set(ha(2),'xlim',[0.4,1],'ylim',[0.4,1]);
for i=1:3
    plot(ha(i),get(ha(i),'xlim'),get(ha(i),'xlim'),'k--')
    xlabel(ha(i),'ideal policy')
    ylabel(ha(i),'oMCD policy')
end




%% subfunctions

function [pi] = myPiStar(dmut,vt,pit,t,gridmu,gridv)
% outputs optimal stopping policy from current value moments
dmu = abs(gridmu-dmut(t+1));
indmu = find(dmu==min(dmu));
dv = abs(gridv-vt(t+1));
indv = find(dv==min(dv));
pi = pit(indmu,indv,t); % ideal policy
end

function [mu,sigma,lambda] = myBayesianUpdate(mu0,sigma0,Sigma,y)
% simple Bayesian update of moments of the value representation
sigma = 1./((1/sigma0)+(1/Sigma));
lambda = 1./((Sigma/sigma0)+1);
mu = mu0 + lambda.*(y-mu0);
end