function [pit,pi0,gridmu,gridv,Pdmu,Pv] = get_pi_AttributeIntegration(R,alpha,nu,eta,zeta,w)
% Derive MDP-optimal stopping rule for attribute integration
% function [pit,pi0] = get_pi_AttributeIntegration(R,alpha,nu,eta,zeta,w)
% This function derives the optimal policy pi*(t) for the discounted
% benefit of effort investment, given parameters that control decision
% confidence and effort cost.
% IN:
%   - R: decision importance
%   - alpha: unitary effort cost
%   - nu:  power of effort cost function
%   - eta: means of attribute distributions
%   - zeta: variances of attribute distributions
%   - w: attribute weights
% OUT:
%   - pit: nmxnvxT RD matrix of ideal policy
%   - pi0: nmxnv matrix of ideal policy at t=0

% initialize parameters
k = size(eta,1); % nb of attributes
v0 = (w.^2)'*zeta; % prior variance over option values
kappa = 1/k; % effort intensity
T = k; % temporal horizon

% define numerical grids for dmu(t) and v(t)
gridmu = linspace(-4*sqrt(2*v0),4*sqrt(2*v0),5e2);
nmu = length(gridmu);
minv = min((w.^2).*zeta);
gridv = linspace(minv,v0,1e2);
nv = length(gridv);

% Confidence landscape Pc(dmu,v)
Pc = NaN(nmu,nv);
for imu=1:nmu % loop over dmu(t)
    dmu = gridmu(imu); % dmu(t)
    for iv=1:nv % loop over v(t)
        v = gridv(iv); % v(t)
        Pc(imu,iv) = getPc_dmu(0,0,v,dmu,0,0); % Pc(dmu(t),v(t))
    end
end

% Transition density: P(dmu(t)|dmu(t-1))
Pdmu = NaN(nmu,nmu);
for i=1:nmu % loop over dmu(t-1)
    thismu = gridmu(i); % dmu(t1)
    Pdmu(i,:) = exp(-0.5*(gridmu-thismu).^2./(4*v0*kappa)); % gaussian density
    Pdmu(i,:) = Pdmu(i,:)./sum(Pdmu(i,:)); % normalize transition density
end

% Transition density:  P(v(t)|v(t-1))
Pv = zeros(nv,nv);
for i=1:nv % loop over v(t-1)
    thisv = gridv(i); % v(t-1)
    nextv = thisv - (w.^2).*zeta; % v(t)
    for ik=1:k % loop over admissible variance decrements
        d0 = abs(nextv(ik)-gridv);
        i0 = find(d0==min(d0)); % index of v(t) on the grid
        Pv(i,i0) = Pv(i,i0)+1;
    end
    Pv(i,:) = Pv(i,:)./sum(Pv(i,:)); % normalize transition density
end

% initialize optimal policy
pit = NaN(nmu,nv,T); % optimal policy on the {dmu(t),v(t)} space

% Initiate backward induction: t=k
EQT = R - alpha.*(kappa*T)^nu; % expected net benefit at t=T
pit(:,:,T) = 0; % stop no matter what at t=T

% First backward induction step: t=k-1
Qt = R*Pc - alpha.*(kappa*(T-1))^nu; % net benefit at t=T-1
pit(:,:,T-1) = (Qt<=EQT); % optimal policy at t=k-1
Qstar = max(cat(3,Qt,EQT*ones(nmu,nv)),[],3); % optimal net benefit at t=k-1
EQstar = (Pv*(Pdmu*Qstar)')'; % Expected optimal net benefit at t=k-1


% loop backward in time
for t=1:(T-2)
    % MDP: backward induction
    Qt = R*Pc - alpha.*(kappa*(T-t-1))^nu; % net benefit at k-t-1
    pit(:,:,T-t-1) = (Qt<=EQstar); % optimal policy at k-t-1
    Qstar = max(cat(3,Qt,EQstar),[],3); % optimal benefit at k-t-1
    EQstar = (Pv*(Pdmu*Qstar)')'; % E[optimal net benefit at k-t]
end

% At t=0
Qt = R*Pc; % net benefit at t=0
pi0 = (Qt<=EQstar); % optimal policy at t=0


