function [w,wP,wdmu] = get_wMDP_BayesianValueDenoising(R,alpha,Sigma,kappa,nu,v0)
% Derive MDP-optimal stopping rule for oMCD
% function [w,dms] = get_wMDP0(R,alpha,beta,gamma,kappa,nu,v0,T)
% This function derives the optimal threshold w(t) for the discounted
% benefit of effort investment, given parameters that control decision
% confidence and effort cost.
% IN:
%   - R: decision importance
%   - alpha: unitary effort cost
%   - beta: type#1 effort efficiency
%   - gamme: type #2 effort efficiency
%   - kappa: effort intensity (per unit of time)
%   - nu: power of effort cost function
%   - v0: initial value variance
% OUT:
%   - w: 1xT vector of discounted benefit thresholds
%   - wP: 1xT vector of confidence thresholds

% check temporal horizon
T = ceil(1/kappa);
if T<=1
    disp('Error: T<=1. Optimal threshold cannot be derived!')
    w = [];
    wP = [];
    wdmu = [];
    return
end

% initialize dmu grid
vdT = T*(Sigma+T*v0)./(((Sigma/v0)+T).^2);
maxdmu = 4*sqrt(2*vdT); % bound = 4 standard deviations
ddmu = maxdmu*1e-3;
dmut = vec(-maxdmu:ddmu:maxdmu);
i0 = find(dmut==0);

% initialize oMCD variables
w = NaN(1,T); % discounted benefit threshold
wP = NaN(1,T); % confidence threshold
wdmu = NaN(1,T); % |dmu| threshold
EQ = NaN(length(dmut),T); % expected optimal discounted benefit

% Thresholds at t=T
w(T) = R/2 - alpha*(kappa*T)^nu; % minimum discounted benefit
wdmu(T) = 0; % minimum value difference
wP(T) = 0.5; % minimum confidence

% Initialize E[Q(T)|dmu(T-1)]
vT = 1/((1/v0)+T/Sigma); % value variance at t=T
Q0T = R*getPc_dmu(0,0,vT,dmut,0,0) - alpha*(kappa*T)^nu;
Pt = myTransitionMatrix(dmut,T-1,Sigma,v0);
EQ(:,T) = Pt*Q0T;

% discounted benefits' threshold: backward induction
for t=1:T-1 % loop through decision times (backward)
    % evaluate discounted benefit of stopping now
    vt = 1/((1/v0)+(T-t)/Sigma); % value variance at T-t
    Pct = getPc_dmu(0,0,vt,dmut,0,0); % confidence at T-t
    Q0t = R*Pct - alpha*(kappa*(T-t))^nu; % value of stopping at T-t
    % find critical point
    iu = find(Q0t(i0:end)>=EQ(i0:end,T-t+1),1); % crossing index
    % evaluate thresholds
    if ~isempty(iu) % if the crossing happens whithin the dmu grid
        w(T-t) = Q0t(i0+iu-1); % discounted benefit threshold
        wP(T-t) = Pct(i0+iu-1); % confidence threshold
        wdmu(T-t) = dmut(i0+iu-1); % |dmu| threshold
    else % the crossing must happen outside the dmu grid
        w(T-t) = R - alpha*kappa*(T-t); % set to max[discounted benefit]
        wP(T-t) = 1; % set to max[confidence]
        wdmu(T-t) = NaN; 
    end
    % recurrent backward induction on E[Q]
    Pt = myTransitionMatrix(dmut,T-t-1,Sigma,v0);
    EQ(:,T-t) = Pt*max([Q0t,EQ(:,T-t+1)],[],2); % Bellman recurrence equation
end

% confidence threshold
% wP0 = (w+alpha*(kappa*(1:T)).^pow)./R;
% vt = 1./((1/v0)+(1:T)./Sigma);
% wP0 = getPc_dmu(0,0,vt,wdmu,0,0);
% figure,plot(wP),hold on,plot(wP0,'r--')


function Pt = myTransitionMatrix(dmut,t,Sigma,v0)
vt = 1/((1/v0)+(t/Sigma)); % value variance at t=T-1
Ed2t = (Sigma+vt)./(((Sigma/vt)+1).^2); % E[dmu(t)]
% transition matrix: P(dmu(t)|dmu(t-1))
Pt = NaN(length(dmut),length(dmut));
for i=1:length(dmut)
    Pt(i,:) = exp(-0.5*(dmut'-dmut(i)).^2./(2*Ed2t));
    Pt(i,:) = Pt(i,:)./sum(Pt(i,:));
end


