function [w,wdmu] = get_wMDP_Tajima(alpha,gamma,kappa,nu)
% Derive MDP-optimal stopping rule of Tajima et al. (2016)
% function [w,wdmu] = get_wMDP_Tajima(alpha,gamma,kappa,pow,T)
% This function derives the optimal threshold w(t) for the discounted
% benefit of effort investment, given parameters that control decision
% confidence and effort cost.
% IN:
%   - alpha: unitary effort cost
%   - gamme: type #2 effort efficiency
%   - kappa: effort intensity (per unit of time)
%   - nu: power of effort cost function
% OUT:
%   - w: 1xT vector of discounted benefit thresholds
%   - wdmu: 1xT vector of value mode difference thresholds

% check temporal horizon
T = ceil(1/kappa);
if T<=1
    disp('Error: T<=1. Optimal threshold cannot be derived!')
    w = [];
    wdmu = [];
    return
end

% transition matrix: P(dmu(t)|dmu(t-1))
maxdmu = 4*sqrt(2*gamma*kappa*T); % bound = 4 standard deviations
ddmu = maxdmu*1e-3;
dmut = vec(-maxdmu:ddmu:maxdmu);
i0 = find(dmut==0);
Pt = NaN(length(dmut),length(dmut));
for i=1:length(dmut)
    Pt(i,:) = exp(-0.5*(dmut'-dmut(i)).^2./(4*gamma*kappa));
    Pt(i,:) = Pt(i,:)./sum(Pt(i,:));
end

% initialize oMCD variables
w = NaN(1,T); % discounted benefit threshold
EQ = NaN(length(dmut),T); % expected optimal discounted benefit

% Thresholds at t=T
w(T) = - alpha*(kappa*T)^nu; % minimum discounted benefit
wdmu(T) = 0; % minimum difference in value modes

% Initialize E[Q(T)|dmu(T-1)]
EQ(:,T) = myEBenefit(dmut,gamma,kappa) - alpha*(kappa*T)^nu;

% discounted benefits' threshold: backward induction
for t=1:T-1 % loop through decision times (backward)
    Q0t = myEBenefit(dmut,0,0) - alpha*(kappa*(T-t))^nu; % net benefit (stop now)
    iu = find(Q0t(i0:end)>=EQ(i0:end,T-t+1),1); % crossing index
    % evaluate thresholds
    if ~isempty(iu) % if the crossing happens whithin the dmu grid
        w(T-t) = Q0t(i0+iu-1); % optimal net value threshold
        wdmu(T-t) = dmut(i0+iu-1); % optimal dmu threshold
    else % the crossing must happen outside the dmu grid
        w(T-t) = NaN;
        wdmu(T-t) = NaN;
    end
    % recurrent backward induction on E[Q]
    EQ(:,T-t) = Pt*max([Q0t,EQ(:,T-t+1)],[],2); % Bellman recurrence equation
end



function Emax = myEBenefit(dmu,gamma,kappa)
% derive E[max(mu1,mu2)] - 05*E[mu1+mu2]
gamkap = gamma*kappa;
if gamkap>0
    tmp = abs(dmu)./sqrt(2*gamkap);
    Emax = 0.5*abs(dmu).*(1-2*normcdf(-tmp)) + sqrt(gamkap./(2*pi)).*exp(-0.5*tmp.^2);
else
    Emax = 0.5*abs(dmu);
end
